// |$Id: bagc.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bagc.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: Automatic Gain Control (AGC)                      |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |  onepole    ARMA operator for single pole on real axis     |
   |  c_boxit    box car filter, add/subtract trick             |
   |  fnorm      normalize a signal with L2 norm or scale factor|
   |  fmax_min   find maximum and minimum values in a signal    |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        int *itype, float *twide, char infile[],
        char ofile[], char lstfile[], char pid[]);

      void outlst(int itype, float twide, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="bagc",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[13];
      struct header     hd;

//...specific to this application
      int               itype,
                        iwide,
                        isign,
                        ntimes,
			kmx,
			kmn;
      float             twide,
                        fmax,
                        fmin,
                        fscal,
                       *s3;

//...get parameters
      if(getparm(argc,argv,&itype,
         &twide,infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(itype,twide,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }


//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          puts("ABORT:s1 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          puts("ABORT:s2 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_s1;
         }

        s3=calloc(npts,sizeof(float));
          if(s3==NULL)
         {
          puts("ABORT:s3 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_s2;
         }


//...set specific parameters
      iwide=twide/fsamin;
      iwide=(iwide/2)*2+1;
      isign=-1;
      ntimes=1;

//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        puts("ABORT--c_bsegin");
        goto quit_it;
       }

//...compute energy envelope
      for (jt=0;jt<npts;jt++) s2[jt]=s1[jt]*s1[jt];

//...smooth energy envelope
      if(itype==1 || itype==3)
      {if(c_boxit(npts,iwide,isign,ntimes,s2,s3)!=0) goto quit_it;}
      else
      {if(onepole(npts,fsamin,twide,s2,s3)!=0) goto quit_it;}
      //endif

//...convert smoothed envelope to amplitude
      for(jt=0;jt<npts;jt++)
       if(s3[jt]!=0.) s3[jt]=sqrt(s3[jt]);

//...find envelope maximum
      fmax_min(npts,s3,&fmax,&fmin,0,&kmx,&kmn);

//...compute output trace: divide by energy envelope or assign
      for(jt=0;jt<npts;jt++)
     {
      if(itype<=1)
       {
        if(s3[jt]!=0.)
         s2[jt]=s1[jt]/(s3[jt]/fmax);
        else
         s2[jt]=0.;
       } //endif
      else
       {
         s2[jt]=s3[jt]/fmax;
       } //endif
     } //endfor

//...normalize by (L2 norm / npts)
        fscal=0.;
        if(fnorm(npts,&fscal,s2,s3)!=0) goto quit_it;
        fscal=1.0/sqrt((float)npts);
        if(fnorm(npts,&fscal,s3,s2)!=0) goto quit_it;

//...zero out first sample to avoid noise spike
	s2[0]=0.0 ;

//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        puts("ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
        return(0);
quit_it:
        puts(" ");
        free(s3);
exit_s2:
        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
        return(1);
     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *itype, float *twide,char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);
      
//...help pannel
      sprintf(msg1,"Automatic Gain Control AGC                   ");
      sprintf(msg2,"Choice of Box Car or Single Pole Envelope    ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile twide itype",pid);
      puts(" ");
      puts("     infile   =  input file name");
      puts("     twide    =  width moving energy window (s)  ");
      puts("     itype    =  envelope smoother and output  ");
      puts("             0=  ARMA one-pole exp. decay ");
      puts("             1=  zero-phase box car ");
      puts(" ");
      puts("             2=  output ARMA envelope ");
      puts("             3=  output BOXCAR envelope ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strcpy(infile,argv[1]);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to float
      if(argc>2)
       *twide=atof(argv[2]);
       else
      {
       puts(" twide:       =  width moving energy window (s)  ");
       puts(" ");
       puts(" Enter twide (float)");
       scanf(" %f",twide);
      }

//...convert string to integer
      if(argc>3)
       *itype=atoi(argv[3]);
       else
      {
       puts(" itype:      0=  ARMA one-pole exp. decay ");
       puts("             1=  zero-phase box car ");
       puts(" ");
       puts("             2=  output ARMA envelope ");
       puts("             3=  output BOXCAR envelope ");
       puts(" ");
       puts(" Enter itype (int)");

       scanf(" %d",itype);
      }

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/ 
      void outlst(int itype, float twide,char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fputs("\n itype:      0=  ARMA one-pole exp. decay ",h3);
      fputs("\n             1=  zero-phase box car ",h3);
      fputs("\n ",h3);
      fputs("\n             2=  output ARMA envelope ",h3);
      fputs("\n             3=  output BOXCAR envelope ",h3);
      fputs(" ",h3);
      fprintf(h3,"\n     itype     = %5.5d ",itype);
      fprintf(h3,"\n     twide (s) = %7.3f ",twide);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
