// |$Id: bbal.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bbal.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: balance the amplitudes of two data sets           |
   |          so that each has the same MAV (Mean               |
   |          Absolute Value)                                   |
   |                                                            |
   |          new MAV for both=(MAV1+MAV2)/2                    |
   |                                                            |
   |NOTE:  Both data sets should have the same number of samples|
   |(actually, same number of traces, same number of samples/tr)|
   |                                                            |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |  mav        compute mean absolute value of a signal        |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         int *iopt, char infile1[], char infile2[], char ofile1[],
         char ofile2[], char lstfile[],char pid[]);

      void outlst(int iopt, char infile1[], char infile2[],
        char pid[], char ofile1[], char ofile2[],
        char lstfile[], FILE *h3);

      float mav(int npts,float s1[]);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3, *h4, *h5;
      int               jt,
                        jrec,
                        npts1, npts2,
                        ntraces1, ntraces2,
                        ibar,
                        invbar;
      float             fsamin1, fsamin2,
                        bar,
                        *s1,
                        *s2,
                        *s3,
                        *s4;
      char              pid[5]="bbal",   //process id name
                        ofile1[13], ofile2[13],
                        lstfile[13],
                        infile1[14], infile2[14];
      struct header     hd1,
                        hd2;

//...specific to this application
      int               iopt,
                        itest;
      float             mav1,
                        mav2,
                        avmav,
                        sfact1,
                        sfact2;

//...get parameters
      if(getparm(argc,argv,&iopt,infile1,infile2,
         ofile1,ofile2,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(iopt,infile1,infile2,pid,ofile1,ofile2,lstfile,h3);

//...open input file 1
        if( (h1=fopen(infile1,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile1);
          goto exit_3;
        }

//...open input file 2
        if( (h4=fopen(infile2,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile2);
          goto exit_1;
        }

//...open output file 1
        if( (h2=fopen(ofile1,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile1);
          goto exit_2;
        }

//...open output file 2
        if( (h5=fopen(ofile2,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile2);
          goto exit_4;
        }

//...check input 1 file size
          if(in_chk(&ntraces1,&npts1,&fsamin1,hd1,h1)!=0)
         {
          goto exit_5;
         }

//...check input 2 file size
          if(in_chk(&ntraces2,&npts2,&fsamin2,hd2,h4)!=0)
         {
          goto exit_5;
         }

//...print input 1 file data to lstfile
        fputs("\n   INPUT FILE_1:",h3);
        fprintf(h3,"\n     Number of samples= %d",npts1);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin1);
        fprintf(h3,"\n     Number of traces = %d",ntraces1);

//...print input 2 file data to lstfile
        fputs("\n  ",h3);
        fputs("\n   INPUT FILE_2:",h3);
        fprintf(h3,"\n     Number of samples= %d",npts2);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin2);
        fprintf(h3,"\n     Number of traces = %d",ntraces2);

//...test for same size
        itest=1;
        if(npts1!=npts2) itest=0;
        if(ntraces1!=ntraces2) itest=0;
        if(fsamin1!=fsamin2) itest=0;
        if(itest==0)
         {puts("\n  ABORT !! Files of different size");
          goto exit_5;
         }

//...dynamic allocation of seimic trace memory
        s1=calloc(npts1,sizeof(float));
          if(s1==NULL)
         {
          puts("ABORT:s1 can't allocate enough trace memory");
          printf(" npts1 = %d",npts1);
          goto exit_5;
         }
        s2=calloc(npts1,sizeof(float));
          if(s2==NULL)
         {
          puts("ABORT:s2 can't allocate enough trace memory");
          printf(" npts1 = %d",npts1);
          goto exit_s1;
         }
        s3=calloc(npts2,sizeof(float));
          if(s3==NULL)
         {
          puts("ABORT:s3 can't allocate enough trace memory");
          printf(" npts2 = %d",npts2);
          goto exit_s2;
         }
        s4=calloc(npts2,sizeof(float));
          if(s4==NULL)
         {
          puts("ABORT:s4 can't allocate enough trace memory");
          printf(" npts2 = %d",npts2);
          goto exit_s3;
         }

//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces1,&bar,&ibar,&invbar);

//...Select Mode: trace or profile
    if(iopt==1)                              //  TRACE MODE
   {
//...trace loop

//...listing header
      fputs("\n\n Trace   mav1       mav2      avmav  ",h3);
      fputs("   sfact1     sfact2\n",h3);
      for (jrec=0;jrec<ntraces1;jrec++)
     {

//...read trace input_1
        if(c_bsegin(jrec,s1,npts1,&hd1,h1)!=0)
       {
        puts("ABORT--c_bsegin 1");
        goto quit_it;
       }

//...read trace input_2
        if(c_bsegin(jrec,s3,npts2,&hd2,h4)!=0)
       {
        puts("ABORT--c_bsegin 2");
        goto quit_it;
       }

//...compute MAV values and scale factors
        mav1=mav(npts1,s1);
        mav2=mav(npts2,s3);
        avmav=(mav1+mav2)/2.0;
        sfact1=avmav/mav1;
        sfact2=avmav/mav2;

//...list factors
        fprintf(h3,"\n %4.4d %10.4E %10.4E %10.4E %10.4E %10.4E",
        jrec+1,mav1,mav2,avmav,sfact1,sfact2);

//...apply scale factors
         for (jt=0;jt<npts1;jt++)
        {
         s2[jt]=s1[jt]*sfact1;
         s4[jt]=s3[jt]*sfact2;
        }

//...output trace 1
        if(c_bsegout(jrec,s2,npts1,&hd1,h2)!=0)
       {
        puts("ABORT--c_bsegout 1");
        goto quit_it;
       }

//...output trace 2
        if(c_bsegout(jrec,s4,npts2,&hd2,h5)!=0)
       {
        puts("ABORT--c_bsegout 2");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces1,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------
   } //end iopt=1 TRACE MODE

   else

   {
//...trace loop                              PROFILE MODE

//...listing header
      fputs("\n\n Trace   mav1       mav2      avmav  ",h3);
      fputs("   sfact1     sfact2\n",h3);

      mav1=0;      // initialize mav accumulators
      mav2=0;

      for (jrec=0;jrec<ntraces1;jrec++)
     {

//...read trace input_1
        if(c_bsegin(jrec,s1,npts1,&hd1,h1)!=0)
       {
        puts("ABORT--c_bsegin 1");
        goto quit_it;
       }

//...read trace input_2
        if(c_bsegin(jrec,s3,npts2,&hd2,h4)!=0)
       {
        puts("ABORT--c_bsegin 2");
        goto quit_it;
       }

//...accumulate MAV values
        mav1=mav1+mav(npts1,s1);
        mav2=mav2+mav(npts2,s3);

     } //  end trace loop-----------------------------

//...compute scale factors for profile mode
        avmav=(mav1+mav2)/2.0;
        sfact1=avmav/mav1;
        sfact2=avmav/mav2;

      for (jrec=0;jrec<ntraces1;jrec++)   //start trace loop2
     {

//...read trace input_1
        if(c_bsegin(jrec,s1,npts1,&hd1,h1)!=0)
       {
        puts("ABORT--c_bsegin 1");
        goto quit_it;
       }

//...read trace input_2
        if(c_bsegin(jrec,s3,npts2,&hd2,h4)!=0)
       {
        puts("ABORT--c_bsegin 2");
        goto quit_it;
       }

//...list factors
        fprintf(h3,"\n %4.4d %10.4E %10.4E %10.4E %10.4E %10.4E",
        jrec+1,mav1,mav2,avmav,sfact1,sfact2);

//...apply scale factors
         for (jt=0;jt<npts1;jt++)
        {
         s2[jt]=s1[jt]*sfact1;
         s4[jt]=s3[jt]*sfact2;
        }

//...output trace 1
        if(c_bsegout(jrec,s2,npts1,&hd1,h2)!=0)
       {
        puts("ABORT--c_bsegout 1");
        goto quit_it;
       }

//...output trace 2
        if(c_bsegout(jrec,s4,npts2,&hd2,h5)!=0)
       {
        puts("ABORT--c_bsegout 2");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces1,bar,ibar,invbar,jrec+1);

     } //  end trace loop2-----------------------------

   } //end iopt=0 PROFILE MODE

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile1);
      printf("\n   Output  ====>%s",ofile2);
      printf("\n   Processed %d traces 1",ntraces1);
      printf("\n   Processed %d traces 2",ntraces2);
      return(0);
	      
quit_it:
        puts(" ");

        free(s4);
exit_s3:
        free(s3);
exit_s2:
        free(s2);
exit_s1:
        free(s1);
exit_5:
        fclose(h5);
exit_4:
        fclose(h2);
exit_2:
        fclose(h4);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
      return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *iopt, char infile1[], char infile2[], char ofile1[],
         char ofile2[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Balances two data sets so that they have     ");
      sprintf(msg2,"the same Mean Absolute Value (MAV)           ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s    infile1   infile2   iopt   \n",pid);
      puts(" ");
      puts("     infile1   =  input file_1 name");
      puts("     infile2   =  input file_2 name");
      puts("        iopt  0=  profile mode ");
      puts("        iopt  1=  trace   mode ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters

//...first input file
      if(argc>1)
       strncpy(infile1,argv[1],13);
       else
      {
       puts(" Enter INPUT file_1 name ");
       scanf(" %13s",infile1);
      }
	if (namecheck(12,infile1) != 0 ) return (1);

//...second input file
      if(argc>2)
       strncpy(infile2,argv[2],13);
       else
      {
       puts(" Enter INPUT file_2 name ");
       scanf(" %13s",infile2);
      }
	if (namecheck(12,infile2) != 0 ) return (1);

//...convert string to integer
      if(argc>3)
       *iopt=atoi(argv[3]);
       else
      {
       puts("  iopt    0=profile mode");
       puts("          1=trace   mode");
       puts("  ");
       puts(" Enter option, iopt (int)");
       scanf(" %d",iopt);
      }

//...name output files
      names(infile1,pid,ofile1,sfx1);
      names(infile2,pid,ofile2,sfx1);
      names(infile1,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int iopt, char infile1[], char infile2[],
        char pid[], char ofile1[], char ofile2[],
        char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input_1   File= %s",infile1);
      fprintf(h3,"\n     Input_2   File= %s",infile2);

      fprintf(h3,"\n     Output_1  File= %s",ofile1);
      fprintf(h3,"\n     Output_2  File= %s",ofile2);

      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     iopt= %5.5d ",iopt);

          if(iopt==0)
           fputs("\n     Profile Mode",h3);
          else
           fputs("\n     Trace Mode",h3);

      fputs("\n",h3);                      //blank line
     }  //  end outlst
