// |$Id: bnfd.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bnfd.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | Compute near field and far field response                  |
   | seismograms from Aki and Richards, Equ 4.23                |
   | (pg 73, vol 1)                                             |
   | NOTE:  This is for a point source in a homogeneous         |
   |        whole space (no source area, no variations          |
   |        in geology)                                         |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  wavelet    exponentially decaying sinusoid                |
   |  findex     find nearest samples for integral limits       |
   |  simpsn     perform integral by Simpson's rule             |
   |  nfield     compute near field term of displacement field  |
   |  ffieldp    compute far field p-wave term of displ.field   |
   |  ffields    compute far field s-wave term of displ.field   |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        int *xforce, float *vp, float *vs, float *den,
	float *alpha, float *fc, int *icomp, int *ifield, char infile[],
        char ofile[], char lstfile[], char pid[]);

      void outlst(int xforce, float vp, float vs, float den,
	float alpha, float fc, int icomp, int ifield, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int wavelet(float alpha, float fc, float fsamin, int npts,
	float wtmax, float xw[], float tm[], int *nw );

      void findex(float x[], float target, int npts, int *kindex);

      void simpsn(float xw[],float tm[], int npts, float fsamin,
	int k1, int k2, float yint[] );

      void nfield(float yint[], int npts,
	float g[], float gf, float df[], int icomp,
        float den, float range,float s2[]);

      void ffieldp(float xw[], int npts,
	float g[], float gf, int icomp, int k1,
        float den, float vp, float range, float s2[]);

      void ffields(float xw[], int npts,
	float g[], float gf, int icomp, int k2, float df[],
        float den, float vs, float range, float s2[]);


      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="bnfd",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      int               xforce;
      int		nw;
      int		k1,k2;
      int		icomp,ifield;
      float		sx,sy,sz,sd,
			rx,ry,rz,range; //coordinates of shot and rec.
      float		vp,vs,den;	//material properties vel and density
      float		alpha,fc;	//wavelet parameters exp decay sine
      float		*xw,
			*tm,		//wavelet and time vector	
			*yint;		//vector for integral, near field term
      float		xpos[3];
      float		g[3],gf;
      float		df[3];
      float		wtmax;
      float		target;	
      float		pi=3.1415926;
      float		afnf,affp,affs;  //amplitude factors, directivity

//...get parameters
      if(getparm(argc,argv,
         &xforce,&vp,&vs,&den,&alpha,&fc,&icomp,&ifield,infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(xforce,vp,vs,den,alpha,fc,icomp,ifield,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }


//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
        xw=calloc(npts,sizeof(float));
          if(xw==NULL)
         {
          fprintf(stderr,"ABORT:xw can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s2;
         }
        tm=calloc(npts,sizeof(float));
          if(tm==NULL)
         {
          fprintf(stderr,"ABORT:tm can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_xw;
         }

        yint=calloc(npts,sizeof(float));
          if(yint==NULL)
         {
          fprintf(stderr,"ABORT:yint can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_tm;
         }


//compute delta function for this problem
	if (xforce==0) df[0]=1; else df[0]=0;
	if (xforce==1) df[1]=1; else df[1]=0;
	if (xforce==2) df[2]=1; else df[2]=0;

//...compute wavelet
	wtmax=(npts-1)*fsamin;
        if(wavelet(alpha,fc,fsamin,npts,wtmax,xw,tm,&nw)!=0)
	{fprintf(stderr,"ABORT--wavelet");
	goto quit_it;}

//...write headers for listing
	fprintf(h3,"\n ");
	fprintf(h3,"\n Trace         ");
	fprintf(h3,"Position vector                             ");
	fprintf(h3,"Direction Cosines                  ");
	fprintf(h3,"Radiation Amplitudes           ");
	fprintf(h3,"\n            x1       x2        x3");
	fprintf(h3,"    Range     Angle     ");
	fprintf(h3,"g1       g2        g3   gForce");
	fprintf(h3,"   NearField    Far_P     Far_S ");


//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...find geometry
         findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);

//...calculate relative geometry (source at origin)
//   position vector is xpos pointing to receiver
	xpos[0]=rx-sx;
	xpos[1]=ry-sy;
	xpos[2]=rz-(sz-sd);

//compute direction cosines
	g[0]=xpos[0]/range;
	g[1]=xpos[1]/range;
	g[2]=xpos[2]/range;
	gf=g[xforce];

//...list direction cosines, position vector
        fprintf(h3,"\n  %2.2d    %7.2f  %7.2f  %7.2f| %7.2f  %7.2f| %7.2f  %7.2f  %7.2f  %7.2f",
	jrec+1,xpos[0],xpos[1],xpos[2],range,(180/pi)*acos(g[2]),g[0],g[1],g[2],gf);

//...compute amplitude factors
	afnf=((3*g[icomp]*gf-df[icomp])/(4*pi*den))/(range*range*range);
	affp=((g[icomp]*gf)/(4*pi*den*vp*vp))/range;
	affs=((g[icomp]*gf-df[icomp])/(4*pi*den*vs*vs))/range;
        fprintf(h3,"| %10.3e %10.3e %10.3e",afnf,affp,affs);

//...find sample index for R/vp and R/vs
	target=range/vp;
        findex(tm,target,npts,&k1);
	target=range/vs;
        findex(tm,target,npts,&k2);
	// fprintf(stdout,"\n k1,k2 = %d  %d ",k1,k2);

//...compute the near field integral
        simpsn(xw,tm,npts,fsamin,k1,k2,yint);

//...clear output trace
	for (jt=0; jt<npts; jt++) s2[jt]=0.;

//...add selected fields to output trace
        if (ifield==1 || ifield==3 || ifield== 5 || ifield==7) 
	nfield(yint,npts,g,gf,df,icomp,den,range,s2);

        if (ifield==2 || ifield==3 || ifield== 6 || ifield==7) 
        ffieldp(xw,npts,g,gf,icomp,k1,den,vp,range,s2);

        if (ifield==4 || ifield==5 || ifield== 6 || ifield==7) 
        ffields(xw,npts,g,gf,icomp,k2,df,den,vs,range,s2);

	if (ifield==0)
	{ for (jt=0;jt<npts;jt++) s2[jt]=xw[jt];}

//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------
	fprintf(h3," \n");

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
      return(0);

quit_it:
        puts(" ");

	free(yint);
exit_tm:
	free(tm);
exit_xw:
	free(xw);
exit_s2:
        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
      return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *xforce, float *vp, float *vs, float *den,
	 float *alpha, float *fc, int *icomp, int *ifield, char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Near Field Seismic Radiation: Homogeneous    ");
      sprintf(msg2,"Whole Space (Aki and Richards Eq 4.23,Vol. I)");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile xforce vp vs den alpha fc icomp ifield\n",pid);
      puts(" ");
      puts("     infile   =  input file name (sets geometry, tmax, ntraces)");
      puts("     xforce   =  point force direction ");
      puts("             1=  in positive x-axis direction ");
      puts("             2=  in positive y-axis direction ");
      puts("             3=  in positive (down) z-axis direction ");
      puts(" ");
      puts("     vp       =  p-wave velocity  ");
      puts("     vs       =  s-wave velocity  ");
      puts("     den      =  mass density     ");
      puts(" ");
      puts("     alpha    =  exponential decay factor (pos) for wavelet ");
      puts("     fc       =  center frequency of wavelet Hz");
      puts("                 (for example, try 50 for alpha and fc) ");
      puts(" ");
      puts("     icomp    =  component of motion to output ");
      puts("             1=  radial ");
      puts("             2=  transverse ");
      puts("             3=  vertical ");
      puts("     ifield   =  fields to include  ");
      puts("                  (SPN) binary coded ");
      puts("             0=  wavelet only ");
      puts("             1=  near field only ");
      puts("             2=  far p-wave only ");
      puts("             4=  far s-wave only ");
      puts("             3=  near and far p-wave only ");
      puts("             5=  near and far s-wave only ");
      puts("             6=  far p- and  s-wave only ");
      puts("             7=  ALL: far S, far P, Near Field  ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *xforce=atoi(argv[2]);
       else
      {
       puts(" Enter xforce (int)");
       scanf(" %d",xforce);
      }

//...check bounds for xforce
	if (*xforce>3)	*xforce=3;
	if (*xforce<1)	*xforce=1;
	*xforce=*xforce-1;	//C-lang Subscripts

//...convert string to float
      if(argc>3)
       *vp=atof(argv[3]);
       else
      {
       puts(" Enter vp (float)");
       scanf(" %f",vp);
      }

//...convert string to float
      if(argc>4)
       *vs=atof(argv[4]);
       else
      {
       puts(" Enter vs (float)");
       scanf(" %f",vs);
      }

//...convert string to float
      if(argc>5)
       *den=atof(argv[5]);
       else
      {
       puts(" Enter den (float)");
       scanf(" %f",den);
      }

//...convert string to float
      if(argc>6)
       *alpha=atof(argv[6]);
       else
      {
       puts(" Enter alpha (float)");
       scanf(" %f",alpha);
      }

//...convert string to float
      if(argc>7)
       *fc=atof(argv[7]);
       else
      {
       puts(" Enter fc Hz (float)");
       scanf(" %f",fc);
      }

//...convert string to integer
      if(argc>8)
       *icomp=atoi(argv[8]);
       else
      {
       puts(" Enter icomp (int)");
       scanf(" %d",icomp);
      }

//...check bounds for icomp 
	if (*icomp>3)	*icomp=3;
	if (*icomp<1)	*icomp=1;
	*icomp=*icomp-1;	//C-lang Subscripts

//...convert string to integer
      if(argc>9)
       *ifield=atoi(argv[9]);
       else
      {
       puts(" Enter ifield (int)");
       scanf(" %d",ifield);
      }

//...check bounds for ifield 
	if (*ifield>7)	*ifield=7;
	if (*ifield<0)	*ifield=0;

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int xforce,float vp, float vs, float den,
	float alpha, float fc, int icomp, int ifield, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     xforce= %d ",xforce+1);
      fprintf(h3,"\n         vp= %f ",vp);
      fprintf(h3,"\n         vs= %f ",vs);
      fprintf(h3,"\n        den= %f ",den);
      fprintf(h3,"\n      alpha= %f ",alpha);
      fprintf(h3,"\n         fc= %f ",fc);
      fprintf(h3,"\n      icomp= %d ",icomp+1);
      fprintf(h3,"\n     ifield= %d ",ifield);
      fputs("\n",h3);                      //blank line
      fputs("\n",h3);                      //blank line
      fputs("\n   icomp    =  component of motion to output ",h3);
      fputs("\n           1=  radial ",h3);
      fputs("\n           2=  transverse ",h3);
      fputs("\n           3=  vertical ",h3);
      fputs("\n   ifield   =  fields to include  ",h3);
      fputs("\n                (SPN) binary coded ",h3);
      fputs("\n           0=  wavelet only ",h3);
      fputs("\n           1=  near field only ",h3);
      fputs("\n           2=  far p-wave only ",h3);
      fputs("\n           4=  far s-wave only ",h3);
      fputs("\n           3=  near and far p-wave only ",h3);
      fputs("\n           5=  near and far s-wave only ",h3);
      fputs("\n           6=  far p- and  s-wave only ",h3);
      fputs("\n           7=  ALL: far S, far P, Near Field  ",h3);
      fputs("\n",h3);                      //blank line
     }  //  end outlst

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: wavelet.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: computes wavelet for synthetic and time           |
   |          vector.                                           |
   |  xw[]=wavelet nw=number of samples                         |
   |  tm[]=sample time vector wtmax=maxtime                     |
   |------------------------------------------------------------|
*/
      int wavelet(float alpha, float fc, float fsamin, int npts,
	float wtmax, float xw[], float tm[], int *nw )
     {
	int	j;
	float   pi=3.1415926;

//...compute number of samples
	*nw=wtmax/fsamin+1;
	if (*nw>npts) return(1);

//...compute vectors
	for (j=0; j<*nw; j++)
	{
	 tm[j]=j*fsamin;
	 xw[j]=exp(-alpha*tm[j])*sin(2*pi*fc*tm[j]);

	} //next j
	return(0);
     }  //end wavelet
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: findex.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   |  find index of closest sample to R/vp and R/vs for         |
   |  integral limits.                                          |
   |                                                            |
   |------------------------------------------------------------|
*/
      void findex(float x[], float target, int npts, int *kindex) 
     {
	int	j;
	float  tst=1.E+30;

	*kindex=0;
	for (j=0; j<npts; j++)
	 {
	  if( fabs(x[j]-target)<tst )
	   {  tst=fabs(x[j]-target);
	      *kindex=j; }  
	 } //next j

     }  //end findex
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: simpsn.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |  Simpson's rule integration of near field term             |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      void simpsn(float xw[],float tm[], int npts, float fsamin,
	int k1, int k2, float yint[] ) 
     {
	int	j,k;
	float   fval1,fval2,fval3;

	for (j=0; j<npts; j++) yint[j]=0.;
	for (j=1; j<npts; j++)
	 {
		for (k=k1; k<k2-1; k+=3)
		 {
		  if ((j-k)<0) fval1=0. ; else fval1=xw[j-k]*tm[k];
		  if ((j-(k+1))<0) fval2=0. ; else fval2=xw[j-(k+1)]*tm[k+1];
		  if ((j-(k+2))<0) fval3=0. ; else fval3=xw[j-(k+2)]*tm[k+2];
		  yint[j]=yint[j]+(fsamin/3.0)*(fval1+4*fval2+fval3);
		 } //next k
	 }  //next j	
     }  //end simpsn
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: nfield.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |  near field term of displacement field                     |
   |                                                            |
   |------------------------------------------------------------|
*/
      void nfield(float yint[], int npts,
	float g[], float gf, float df[], int icomp,
        float den, float range, float s2[])
     {
	int	j;
	float   pi=3.1415926;
	float	scal;

	scal=(3*g[icomp]*gf-df[icomp])/(4*pi*den);	
	scal=scal/(range*range*range);
	for (j=0; j<npts; j++)
	 {
	 s2[j]=s2[j]+scal*yint[j]; 
	 }  //next j

      } //end nfield
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: ffield.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |  far field pwave term of displacement field                |
   |                                                            |
   |------------------------------------------------------------|
*/
      void ffieldp(float xw[], int npts,
	float g[], float gf, int icomp, int k1,
        float den, float vp, float range, float s2[])
     {
	int	j;
	float   pi=3.1415926;
	float	scal;

	scal=(g[icomp]*gf)/(4*pi*den*vp*vp);	
	scal=scal/(range);
	for (j=0; j<npts; j++)
	 {
	 if ((j-k1) >=0) s2[j]=s2[j]+scal*xw[j-k1]; 
	 }  //next j

      } //end ffieldp
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: ffield.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |  far field swave term of displacement field                |
   |                                                            |
   |------------------------------------------------------------|
*/
      void ffields(float xw[], int npts,
	float g[], float gf, int icomp, int k2, float df[],
        float den, float vs, float range, float s2[])
     {
	int	j;
	float   pi=3.1415926;
	float	scal;

	scal=(g[icomp]*gf-df[icomp])/(4*pi*den*vs*vs);	
	scal=scal/(range);
	for (j=0; j<npts; j++)
	 {
//NOTE: Sign is minus here---------\|/
	 if ((j-k2) >=0) s2[j]=s2[j]-scal*xw[j-k2]; 
	 }  //next j

      } //end ffieldp


