! |$Id: bamx.F90,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
! --------------------------------------------------------------
! | Basic Seismic Utilities (BSU)                              |
! | LANGUAGE: Fortran77                                        |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: bamx.f                                       |
! | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
! | REVISION: Double precision version of plplot               |
! |------------------------------------------------------------|
! | Amplitude analysis program for Surface Waves with offset.  |
! | Corrects for spherical divergence and determines the       |
! | inelastic, exponential amplitude decay.  Output includes   |
! | a history file for decay coefficients as a function of     |
! | frequency. This program is generally used in conjunction   |
! | with bvax.f which determines wave velocity dispersion.     |
! | The combination of bamx.f and bvax.f runs on an offset     |
! | interval may be analyzed under the Kelvin constitutive     |
! | model to determine the soil stiffness and damping          |
! | coefficients.                                              |
! |------------------------------------------------------------|
! |  NOTICE:                                                   |
! |  Copyright (C) 2017 Paul Michaels                          |
! |  <pm@cgiss.boisestate.edu>                                 |
! |  This program is free software; you can                    |
! |  redistribute it and/or modify it under the terms          |
! |  of the GNU General Public License as published            |
! |  by the Free Software Foundation; either version           |
! |  3 of the License, or (at your option) any later           |
! |  version.  This program is distributed in the              |
! |  hope that it will be useful, but                          |
! |  WITHOUT ANY WARRANTY; without even the implied            |
! |  warranty of MERCHANTABILITY or FITNESS FOR A              |
! |  PARTICULAR PURPOSE.  See the GNU General Public           |
! |  License for more details.                                 |
! |  You should have received a copy of the GNU                |
! |  General Public License along with this program;           |
! |  if not, write to the Free Software Foundation,            |
! |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
! |------------------------------------------------------------|
! |  PARAMETERS:                                               |
! |  ndim = maximum number of samples in a seismic trace       |
! |  mdim = maximum number of stations and frequencies         |
! |------------------------------------------------------------|
! |  INCLUDE FILE REQUIRED:                                    |
! |  ../include/bsegy.f90    (modified segy header definitions)|
! |                (no reel header, floating point data)       |
! |------------------------------------------------------------|
! |  LIBRARY REQUIRED:                                         |
! |                                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |------------------------------------------------------------|
! |  SUBROUTINES CALLED:                                       |
! | In-line===============                                     |
! |     GETPRM  get input parameters from command line or      |
! |             interactive query.  Should be inline with      |
! |             main program, specific to application.         |
! |     LSTPRM  echo check of input parameters from GETPRM.    |
! |             Values written to a history file (*.lst)       |
! |     STAGE1  Check geometry and some calculations           |
! |     ZRANGE  Selects data                                   |
! |     APLOT   Postscript plot generation: req. plplot package|
! |     APLOT2  Postscript QC plots of amp: req. plplot package|
! | Located in bsu-2.0.2/src/sublibF4===========               |
! |     CHKTRC  get number of samples, sample interval, and    |
! |             number of traces in the input data file.       |
! |             Also returns values for progress bar, stdout   |
! |     BSEGIN  Reads a seismic trace.                         |
! |     BSEGOUT Write a seismic trace.                         |
! |     FINDXYZ Get x,y,z, coordinates from headers            |
! |                                                            |
! --------------------------------------------------------------
!
!
      PROGRAM main
      include '../include/bsegy.f90'
      parameter (ndim=32767,mdim=1000)
!
      real*4            s1(ndim),s2(ndim)
      real*4            rrange(mdim)
      real*4            svfreq(mdim),svamp(mdim),svvar(mdim)
      real*4            amp(mdim),dist(mdim),ampdb(mdim),distdb(mdim)
      real*4            xt(mdim)
      real*8            xmin,xmax
      character*4       aplc
      character*40      infil, outfil, outlst
      integer*1         bz
      character*1024    cmd
      common /header/ hd
!
!...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
!
!----------------START HARDWIRE OPTIONS--------------
!
!...isw1=0 use peak value  isw1=1 use L2 norm
      isw1=0
!
!...select slope only or slope plus intercept fit
!     islpsw=0 slope only (NOT RECOMMENDED)
!     islpsw=1 slope+intercept (RECOMMENDED) less sensitive to ref. offset
      islpsw=1
!
!...set filter and other constants
      itype=1
!
!...zerophase filters
      ifaz=0
!...minimum phase filters
!     ifaz=1
!
      if(ifaz.eq.1) then
      npoles=14
      else
      npoles=14
      endif
!------------------------END HARDWIRE--------------
!
!...define application prefix
      write(aplc,'(''bamx'')')
!
!...get parameters-------------------------------------------
      write(outfil,'('' '')')
      call GETPRM(nargsx,infil, &
      rmin,rmax,fmin,fmax,delf,bw,tmax1)
!
!...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil, &
      bar,invbar,ibar,ntrace,ndim)
!
!...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil, &
      infil,rmin,rmax,fmin,fmax,delf,bw,tmax1,ifaz,npoles, &
      islpsw,isw1)
!
!...STAGE 1: Load Geometry Tables----------------------------
      icall1=0
!
      call STAGE1(npts,ntrace,s1,rrange,io11,ndim, &
      infil,icall1,iunit1,fsamin,mdim)
!
!...compute rrange increment
      deltar=rrange(2)-rrange(1)
      write(io11,'(''   Range increment= '',f10.2)') deltar
!
!...clear output file...
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
!
      icall1=0
      icall2=0
!
!...need to close unit1 under gfortran.  g77 would permit redefinition
!   of recl on open file, gfortran does not.
      close(iunit1)
!
!...depth of interest
      call ZRANGE(ntrace,rmin,rmax,jmin,jmax,rrange, &
      zmin,zmax,io11)
!
!...compute reference range
      frrange=rrange(jmin)
      kmin=1
      if(frrange.gt.rrange(jmax)) then
      frrange=rrange(jmax)
      kmin=jmax-jmin+1
      endif
      rangem=frrange
      write(io11,'(''   range min='',f10.3//)') rangem
!
!
!...data loop=======================================....
!
!...Set Time Gate for measurements
      jtmin=1
      jtmax=int(tmax1/fsamin+1)
      write(io11,'(''   Jtmax= '',i5)') jtmax
      if(jtmax.gt.npts) then
      jtmax=npts
      write(io11,*) 'maximum time gate reset to npts'
      endif
!
!
!...start FILTER LOOP
      nfilt=int((fmax-fmin)/delf+1)
      freq=fmin-delf
      ilast=0
      do 500, jfilt=1,nfilt
      freq=freq+delf
      fcenter=freq
      bwidth=bw
!
      write(io11,23) fcenter
   23 format (//25(1h-),"START FREQUENCY=",f6.1, &
      25(1h-)) 
      write(io11,*) '  RAW MEASUREMENTS IN WINDOW:'
      write(io11,*) '  index  distance       amp           amp*range'

!
      krec=0
!
      do 100 jrec=jmin,jmax
!
!...input a trace
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
      ntr = ntr + 1
!
      krec=krec+1
!
      if(krec.gt.mdim) then
      write(*,*)' ABORT--mdim dimension exceeded'
      stop
      endif
!
!...filter trace
!
      call ARMA(npts,s1,s2,itype,fcenter,bwidth,npoles,fsamin,ifaz)
!
      if(isw1.eq.0) then
!
!...find peak value for filtered trace in time gate
      amp(krec)=-9999999999.
      do 48 jt=1,jtmax
      if(abs(s2(jt)).gt.amp(krec)) then
      amp(krec)=abs(s2(jt))
      endif
   48 continue
      else
!
!...compute L2 norm for filtered trace in time gate
      sumx=0
      do 50 jt=1,jtmax
      sumx=sumx+s2(jt)*s2(jt)
   50 continue
!
      if(sumx.ne.0.) then
      amp(krec)=sqrt(sumx)
      else
      amp(krec)=0.
      endif
!
      endif
!
!...apply range to cancel cylindrical decay sqrt(1/R)
      dist(krec)=rrange(jrec)
      tmp2=amp(krec)
      amp(krec)=amp(krec)*sqrt(dist(krec))
      write(io11,'(2x,i5,f10.3,2x,e14.7,2x,e14.7)') &
      krec,dist(krec),tmp2,amp(krec)
!
!//////////////////////////////////////////////////////////////////
!
!
!...END LOOP========================================....
  100 continue
      nlevel=krec
!
!...compute amplitude decay in dB (log domain)
      do 110 krec=1,nlevel
      if(islpsw.eq.0) then
      ampdb(krec)=20.*alog10(amp(krec)/amp(kmin))
      distdb(krec)=dist(krec)-rangem
      else
      ampdb(krec)=20.*alog10(amp(krec)/amp(kmin))
      distdb(krec)=dist(krec)
      endif
  110 continue
!
!...compute least squares fit
      fn=float(nlevel)
      sumx=0.
      sumy=0.
      sumxy=0.
      sumx2=0.
      do 120 krec=1,nlevel
      wy=ampdb(krec)
      sumx=sumx+distdb(krec)
      sumy=sumy+wy
      sumxy=sumxy+distdb(krec)*wy
      sumx2=sumx2+distdb(krec)*distdb(krec)
  120 continue
!
!      L2 is standard least squares
!------------------------------------------------
!   L2 Norm
!
!...select slope only or slope plus intercept fit
      if(islpsw.eq.1) then
!
!...solution
      fn=float(nlevel)
      slope=(sumx*sumy-fn*sumxy)/(sumx*sumx-fn*sumx2)
      bcept=(sumxy*sumx-sumy*sumx2)/(sumx*sumx-fn*sumx2)
      fnm=fn-2.
      else
      fn=float(nlevel)
      slope=sumxy/sumx2
      bcept=0.
      fnm=fn-1.
      endif
!
!------------------------------------------------
!...compute error of fit, error bar
      write(io11,'(//'' '')')
      write(io11,*) '     Distance     Observed_dB     Computed_dB  '
      sigmay=0.
      do 130 krec=1,nlevel
      tmp1=distdb(krec)*slope+bcept
      xt(krec)=tmp1
      sigmay=sigmay + (ampdb(krec)-tmp1)**2
      write(io11,'(5x,f6.2,7x,f10.4,6x,f10.4)') &
       distdb(krec),ampdb(krec),tmp1
  130 continue
      sigmay=sigmay/(fnm)
!
      sdevy=sqrt(sigmay)
      write(io11,'(//'' LSQE Fit: S_Dev +/- dB='',f10.4)') sdevy
!
!   L2 Norm
      if(islpsw.eq.1) then
      sigmam=(sigmay*fn)/(fn*sumx2-sumx*sumx)
      else
      sigmam=sigmay/sumx2
      endif
      if(sigmam.ne.0.0) then
      erbarv=sqrt(sigmam)
      else
      erbarv=0.0
      endif
!
!...write postscript plot
      if (jfilt.eq.nfilt) ilast=1
#ifdef PLPLOT
      call aplot(nlevel,distdb,ampdb,xt,slope,bcept,freq, &
      lineid,tyear,tday,ilast,mdim)
#else
      call gaplot(nlevel,distdb,ampdb,xt,slope,bcept,freq, &
      lineid,tyear,tday,ilast)
#endif

!
!...output results
      open(unit=3,file='bamxdb.his',access='append')
      write(3,'(2x,f6.2,2x,f10.4,2x,f10.4)') freq,slope,erbarv
      close(3)
!
      slope2=-slope/8.68589
      erbar2=erbarv/8.68589
!
!...block negative decay
      if(slope2.lt.0.0) slope2=0.
      open(unit=3,file='bamx.his',access='append')
      write(3,'(2x,f6.2,2x,f10.4,2x,f10.4)') freq,slope2,erbar2
      close(3)
!
!...save results for summary plot
      svfreq(jfilt)=freq
      svamp(jfilt)=slope2
      svvar(jfilt)=erbar2
!
      write(io11,'(''  FREQ='',f6.2)') freq
      write(io11,'(''  Amplitude Decay dB/m='',f10.4)') slope
      write(io11,'(''                   +/-='',f10.4)') erbarv
      write(io11,'(''             Intercept='',f10.4)') bcept
      write(io11,'(/''  Alpha (1/m)='',f10.4)') slope2
      write(*,1101) freq,slope,slope2
1101  format('  Freq.=',f6.2,' dB/m=',f6.2,5x, &
      ' 1/m=',f6.2)
!
!...end FILTER LOOP--------------------------------------
  500 continue
#ifndef PLPLOT
      bz=0
      write(cmd,'(''gnuplot bamxqc.gp'',a)') bz
      call runcmd(cmd)
#endif
!
!...generate summary plot
      ilast=1
      xmin=rmin
      xmax=rmax
#ifdef PLPLOT
      call aplot2(nfilt,svfreq,svamp,svvar,xmin,xmax, &
      lineid,tyear,tday,ilast,mdim)
#else
      call gaplot2(nfilt,svfreq,svamp,svvar,rmin,rmax, &
      lineid,tyear,tday)

      bz=0
      write(cmd,'(''gnuplot bamx.gp'',a)') bz
      call runcmd(cmd)
#endif

!
      write(6,240) ntrace,outlst
240   format(' Traces Processed = ',i5/ &
      ' History Files: bamx.his bamxdb.his  ', &
      /' LISTING FILE: ',a)
      write(*,*) 'POSTSCRIPT FILES bamx.ps and bamxqc.ps'
#ifndef PLPLOT
      write(*,*) 'GNUPLOT Files: bamx.gp, bamxqc.gp'
#endif
!
!
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: getprm.f                                     |
! |------------------------------------------------------------|
! | get input parameters or help                               |
! --------------------------------------------------------------

      subroutine GETPRM(nargsx,infil, &
      rmin,rmax,fmin,fmax,delf,bw,tmax1)
!
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
      character*40      arg7,arg8
!
!...get input/output file names...
!   |--------------------------------------------|
!   | file naming convention                     |
!   | application prefix 4 characters "bxxx"     |
!   | data set suffix 4 characters "dddd"        |
!   | file name = bxxxdddd.seg                   |
!   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
!
!cc---LINUX VERSION
      nargsx=iargc()
!...test for help
      karg=-1
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '|bamx: measure amplitude decay as a        |'
      write(*,*) '|function of frequency                     |'
      write(*,*) '|       cylindrical divergence corrected)  |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*) &
      'bamx infile rmin rmax fmin fmax delf bw tmax'
      write(*,*)' '
      write(*,*)'infile  =input file name '
      write(*,*)'rmin    =scan gate: (min_offset) '
      write(*,*)'rmax    =scan gate: (max_offset) '
      write(*,*)'fmin    =min band pass center frequency (Hz) '
      write(*,*)'fmax    =max band pass center frequency (Hz) '
      write(*,*)'delf    =frequency step (Hz) '
      write(*,*)'bw      =bandwidth of filter (Hz) '
      write(*,*)'tmax    =time gate: max_time '
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
!
      if(nargsx.lt.(3+karg)) then
      write(*,*)'scan gate: rmin=(min_range) '
      write(*,*)'  enter: rmin'
      read(*,*) rmin
      else
      call getarg(2,arg2)
      read(arg2,'(f7.0)') rmin
      endif
!
      if(nargsx.lt.(4+karg)) then
      write(*,*)'scan gate: rmax=(max_range) '
      write(*,*)'  enter: rmax'
      read(*,*) rmax
      else
      call getarg(3,arg3)
      read(arg3,'(f7.0)') rmax
      endif
!
      if(nargsx.lt.(5+karg)) then
      write(*,*)'min center frequency (Hz) (float)  '
      write(*,*)'  enter: fmin'
      read(*,*) fmin
      else
      call getarg(4,arg4)
      read(arg4,'(f7.0)') fmin
      endif
!
      if(nargsx.lt.(6+karg)) then
      write(*,*)'max center frequency (Hz) (float)  '
      write(*,*)'  enter: fmax'
      read(*,*) fmax
      else
      call getarg(5,arg5)
      read(arg5,'(f7.0)') fmax
      endif
!
      if(nargsx.lt.(7+karg)) then
      write(*,*)'frequency step (Hz) (float)  '
      write(*,*)'  enter: delf'
      read(*,*) delf
      else
      call getarg(6,arg6)
      read(arg6,'(f7.0)') delf
      endif
!
      if(nargsx.lt.(8+karg)) then
      write(*,*)'bandwidth (Hz) (float)'
      write(*,*)'  enter: bw'
      read(*,*) bw
      else
      call getarg(7,arg7)
      read(arg7,'(f7.0)') bw
      endif
!
      if(nargsx.lt.(9+karg)) then
      write(*,*)'time gate: tmax'
      write(*,*)'  enter: tmax'
      read(*,*) tmax1
      else
      call getarg(8,arg8)
      read(arg8,'(f7.0)') tmax1
      endif
!
      return
      end
!
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: lstprm.f                                     |
! |------------------------------------------------------------|
! | echo check of input parameters, writes to a history file   |
! --------------------------------------------------------------
!
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil, &
      infil,rmin,rmax,fmin,fmax,delf,bw,tmax1,ifaz,npoles, &
      islpsw,isw1)
      character*4       aplc
      character*40      outlst,outfil,infil
!
!...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
!
      write(io11,'('' Program bamx'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
!
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   rmin= '',f10.2)') rmin
      write(io11,'(''   rmax= '',f10.2)') rmax
      write(io11,'(''   fmin= '',f10.2)') fmin
      write(io11,'(''   fmax= '',f10.2)') fmax
      write(io11,'(''   delf= '',f10.2)') delf
      write(io11,'(''   bw  = '',f10.2)') bw
      write(io11,'(''   tmax= '',f10.4)') tmax1
      write(io11,*) ' HARDWIRED PARAMETERS '
      if(ifaz.eq.0) then
      write(io11,*) 'Filters are Zero Phase'
      else
      write(io11,*) 'Filters are Minimum Phase'
      endif
      if(isw1.eq.0) then
      write(io11,*) 'Pick Peak absolute value for amplitude'
      else
      write(io11,*) 'Pick L2 norm of trace for amplitude'
      endif
      write(io11,*) 'L2 Norm used for fit y=a+bx'
      if(islpsw.eq.0) then
      write(io11,*) 'fit y=bx slope only '
      else
      write(io11,*) 'fit y=bx+a slope-intercept'
      endif
      write(io11,'(''  npoles= '',i5)')    npoles
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
!
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: stage1.f                                     |
! |                                                            |
! | STAGE 1: Load Geometry                                     |
! |------------------------------------------------------------|
! |                                                            |
! |                                                            |
! --------------------------------------------------------------
!
      subroutine STAGE1(npts,ntrace,s1,rrange,io11,ndim, &
      infil,icall1,iunit1,fsamin,mdim)
      include '../include/bsegy.f90'
!
      real*4            rrange(*)
!
      real*4            s1(*)
      character*40      infil
      common /header/ hd
!
      jrec = 0
!
!...BEGIN LOOP====================================....
!
!     write(*,*)' '
!     write(*,*)'  STAGE 1: Load geometry....'
!     write(*,'(1x,9(3x,i2.2),2x,i3)') (k,k=10,100,10)
!     write(*,'(1x)')
!
      do 600 jrec=1,ntrace
!
!...input a trace
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
      if(iexit.eq.1) go to 600
!
      call findxyz(sx,sy,sz,sd,rx,ry,rz,frange,rangex)
!
!
!...save offset range for each trace
      if(jrec.gt.mdim) then
      write(*,*)' ABORT--number of traces>mdim dimension'
      stop
      endif
!
      rrange(jrec)=frange
!
!...END LOOP========================================....
600   continue
      write(io11,'(/''   GEOMETRY:''/)')
      write(io11,123)
  123 format(5x,'Trace',5x,'Range'/)
      do 32 jrec=1,ntrace
      write(io11,'(5x,i5,5x,f5.2)') &
      jrec,rrange(jrec)
!
   32 continue
      write(*,*) ' '
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: zrange.f                                     |
! |                                                            |
! |------------------------------------------------------------|
! | Finds offset range based first and last traces             |
! | zmin and zmax correspond to actual values nearest rmin,rmax|
! --------------------------------------------------------------
!
      subroutine ZRANGE(ntrace,rmin,rmax,jmin,jmax,rrange, &
      zmin,zmax,io11)
      real*4            rrange(*)
!
      eps1=1.0E+20
      eps2=1.0E+20
      do 10 j=1,ntrace
      test=abs(abs(rrange(j))-abs(rmin))
      if(test.lt.eps1) then
      jmin=j
      zmin=rrange(j)
      eps1=test
      endif
      test=abs(abs(rrange(j))-abs(rmax))
      if(test.lt.eps2) then
      jmax=j
      zmax=rrange(j)
      eps2=test
      endif
   10 continue
      if(zmin.gt.zmax) then
      hold=zmax
      zmax=zmin
      zmin=hold
      endif
      write(io11,'(''   rmin= '',f8.1,'' Trace#='',i5)') rmin,jmin
      write(io11,'(''   rmax= '',f8.1,'' Trace#='',i5)') rmax,jmax
      write(io11,'(''   zmin= '',f8.1,'' Trace#='',i5)') zmin,jmin
      write(io11,'(''   zmax= '',f8.1,'' Trace#='',i5)') zmax,jmax
!
!...strategy for possible reverse trace order
      if(jmin.gt.jmax) then
      jhold=jmin
      jmin=jmax
      jmax=jhold
      endif
!
      return
      end
#ifdef PLPLOT
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: aplot.f                                      |
! |                                                            |
! |------------------------------------------------------------|
! |REQUIRES PLPLOT:                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |------------------------------------------------------------|
! |                                                            |
! |   x = range in meters                                      |
! |   y = observed amplitude in dB                             |
! |  yc = calculated dB                                        |
! --------------------------------------------------------------
!
      subroutine aplot(npts,x,y,yc,slope,bcept,freq, &
      lineid,tyear,tday,ilast,mdim)
      use plplot
      real*4 x(npts),y(npts),yc(npts)
      real*8 xd(npts),yd(npts)
      real*8 fmargin,aspect,yjust,xjust
      real*8 xmin,xmax,ymin,ymax
      real*8 xmn,xmx,ymn,ymx
      real*8 fdef,fscale,fzero
      real*8 fwidth
!
      character*14 pltfil
      character*80 titl
      character*40 xlbl
      character*40 ylbl
      character*4 lineid
      integer*2 tyear,tday
      data ifirst /1/
!...copy to double
      do 889, j=1,npts
      xd(j)=x(j)
      yd(j)=y(j)
889   continue
!
!...open device
      if (ifirst.eq.1) then
!  Portrait
      izero=0
      ione=1
      call plscolor(izero)
      call plsori(ione)
!...aspect ratio needed for portrait, unless you don't mind
!   plots which are squished in the vertical dimension.
!   aspect=8.5/11.0
          fmargin=0.
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('ps')
          write(pltfil,'(''bamxqc.ps'')')
!
      call plsdidev(fmargin,aspect,yjust,xjust)
!
      call plsfnam(pltfil)
!
!...initialize plot
      nx=1
      ny=1
      izero=0
      call plinit()
      call plssub(nx,ny)
      call plfontld(izero)
      ifirst=0
!
      endif
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 10, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymin=min(ymin,0.0)
      ymax=max(y(j),ymax)
      ymax=max(ymax,60.0)
  10  continue
!
!...set environment, window and viewport
!      call plenv(xmin,xmax,ymin,ymax,0,0)
      izero=0
      ione=1
      iwidth=1
      fwidth=1.0
      fdef=0.
      fscale=0.6
      call pladv(izero)
      call plschr(fdef,fscale)
      call plprec(ione,izero)
      call plssym(fdef,fscale)
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      xmn=0.1
      xmx=.95
      ymn=0.6
      ymx=.95
      call plvpor(xmn,xmx,ymn,ymx)
      call plwind(xmin,xmax,ymin,ymax)
      fzero=0.
      izero=0
      call plbox('bcnst',fzero,izero,'bcnstv',fzero,izero)
!
      write(titl,'('' FREQ='',f6.1,'' Hz (FIT:'', &
      f8.4,'' dB/m,'',f6.1, &
      '' dB) ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       freq,slope,bcept,lineid,tyear,tday
!
      write(xlbl,'(''Range (m) '')')
      write(ylbl,'(''Decibels'')')
      call pllab(xlbl,ylbl,titl)
      icode=9
      call plpoin(xd,yd,icode)
!
!...plot calculated
!...copy to double
      do 899, j=1,npts
899   yd(j)=yc(j)
      call plline(xd,yd)
      if(ilast.eq.1) then
      call plend()
      endif
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: aplot2.f                                     |
! |                                                            |
! |------------------------------------------------------------|
! |REQUIRES PLPLOT:                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |------------------------------------------------------------|
! |                                                            |
! |   x = frequency in hz                                      |
! |   y = amplitude decay after sph.div. correction nepers/m   |
! |  yvar=one standard deviation                               |
! --------------------------------------------------------------
!
      subroutine aplot2(npts,x,y,yvar,xmin,xmax, &
      lineid,tyear,tday,ilast,mdim)
      use plplot
      real*4 x(npts),y(npts),yvar(npts)
      real*8 xd(npts),yd(npts)
      real*8 fdef,fscale
      real*8 fmargin,aspect,yjust,xjust
      real*8 xmin,xmax,ymin,ymax
      real*8 xx,y1,y2
      real*8 fwidth
      character*14 pltfil
      character*80 titl
      character*40 xlbl
      character*40 ylbl
      character*4 lineid
      integer*2 tyear,tday
      data ifirst /1/
!
      izero=0
      ione=1
!...copy to double precision for plot
      do 1105, j=1,npts
      xd(j)=x(j)
      yd(j)=y(j)
1105  continue
!
!...open device
      if (ifirst.eq.1) then
!  Portrait
      call plscolor(izero)
      call plsori(ione)
!...aspect ratio needed for portrait, unless you don't mind
!   plots which are squished in the vertical dimension.
!   aspect=8.5/11.0
          fmargin=0.
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('ps')
          write(pltfil,'(''bamx.ps'')')
!
      call plsdidev(fmargin,aspect,yjust,xjust)
!
      call plsfnam(pltfil)
!
!...initialize plot
      nx=1
      ny=2
      call plinit()
      call plssub(nx,ny)
      call plfontld(izero)
      ifirst=0
!
      endif
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 10, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
  10  continue
!...set standard scales
      xmin=min(xmin,0.)
      ymin=min(ymin,0.)
!     ymax=max(ymax,0.5)
      ymax=0.5
!
!...set environment, window and viewport
      izero=0
      ione=1
      iwidth=1
      fwidth=1.0
      fdef=0.
      fscale=0.9
      call plschr(fdef,fscale)
      call plprec(izero,izero)
      call plssym(fdef,fscale)
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      call plenv(xmin,xmax,ymin,ymax,izero,izero)
!
      write(titl,'(''Decay (95%conf) '', &
      ''Elev. '',f6.2,'' to '', &
      f6.2,'' m ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       xmin,xmax,lineid,tyear,tday
      write(xlbl,'(''Frequency Hz '')')
      write(ylbl,'(''Decay (1/m)'')')
      call pllab(xlbl,ylbl,titl)
      icode=9
      call plpoin(xd,yd,icode)
!
!...errorbars for 95% confidence
!      do 50, j=1,npts
!      xx=xd(j)
!      y1=yd(j)+yvar(j)*1.96
!      y2=yd(j)-yvar(j)*1.96
!      call plerry(1,xx,y1,y2)
!   50 continue
      call plerry(xd,yd+yvar*1.96,yd-yvar*1.96)
!
      if(ilast.eq.1) then
      call plend()
      endif
      return
      end
#else
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: gaplot2.f                                    |
! |                                                            |
! |------------------------------------------------------------|
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |------------------------------------------------------------|
! |                                                            |
! |   x = frequency in hz                                      |
! |   y = amplitude decay after sph.div. correction nepers/m   |
! |  yvar=one standard deviation                               |
! --------------------------------------------------------------
!
      subroutine gaplot2(npts,x,y,yvar,rmin,rmax, &
      lineid,tyear,tday)
      real*4 x(npts),y(npts),yvar(npts)
      character*4 lineid
      integer*2 tyear,tday
      character*80 sxrange,syrange
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 5, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
   5  continue
!...set overides         
      testy=ymax
      xmin=0.0
      ymin=0.0
      if (testy.LT.1.0) ymax=1.0
      if (testy.LT.0.5) ymax=0.5
      if (testy.LT.0.2) ymax=0.2
      if (testy.LT.0.1) ymax=0.1
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax
!
      open(unit=12,file='bamx.gp',access='sequential')
      write(12,*) "# Generated by bamx.F90  Subroutine gaplot2()     "
      write(12,*) "#set terminal x11 persist                         "
      write(12,*) "set terminal postscript size 7.0,5.0 in \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output 'bamx.ps'                              "
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Frequency (Hz)'                       "
      write(12,*) "set ylabel 'Decay (1/meter)                       "
      write(12,*) "set title 'Decay (95% Conf) Offsets: \"
      write(12,'(F6.2," to ",F6.2,"\")') rmin,rmax 
      write(12,*) "meters \"
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " '                                                "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2):($3*1.96) w yerrorbars \"
      write(12,*) "pt 6 ps 1                                         "
      do 10, j=1,npts
   10 write(12,'(3(E14.7,1x))') x(j),y(j),yvar(j)
      write(12,*) "e "
      close(12)
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: gaplot.f                                     |
! |                                                            |
! |------------------------------------------------------------|
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |------------------------------------------------------------|
! |                                                            |
! |   x = range in meters                                      |
! |   y = observed amplitude in dB                             |
! |  yc = calculated dB                                        |
! --------------------------------------------------------------
!
      subroutine gaplot(npts,x,y,yc,slope,bcept,freq, &
      lineid,tyear,tday,ilast)
      real*4 x(npts),y(npts),yc(npts)
      real*4 ybar(2),xbar(2)
      character*4 lineid
      integer*2 tyear,tday
      character*80 sxrange,syrange
      data ifirst /1/
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 5, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
    5 continue
!...overides
      testy=ymax
      if (testy.LT.60.0) ymax=60.
      if (testy.LT.20.0) ymax=20.
      if (testy.LT.10.0) ymax=10.
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax
!--------test if first call-------
      if (ifirst.eq.1) then     
      open(unit=12,file='bamxqc.gp',access='sequential')
      write(12,*) "# Generated by bamx.F90  Subroutine gaplot()      "
      write(12,*) "set terminal postscript size 7.0,5.0 in \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output 'bamxqc.ps'                            "
      ifirst=0
      endif
!------end ifirst test------------
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Range (m)'                            "
      write(12,*) "set ylabel 'Decibels'      "
      write(12,*) "set title 'Frequency=\"
      write(12,'(f6.1," Hz FIT:=",f8.4,"dB/m, ",f6.1," dB\")') &
       freq,slope,bcept
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " '  "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) w p pt 6 ps 1, \"
      write(12,*) " '' u ($1):($2) w l ls 1     "
      do 10, j=1,npts
   10 write(12,'(2(E14.7,1x))') x(j),y(j)
      write(12,*) "e "
!...plot calculated
      xbar(1)=x(1)
      xbar(2)=x(npts)
      ybar(1)=yc(1)
      ybar(2)=yc(npts)
      do 20, j=1,2
   20 write(12,'(2(E14.7,1x))') xbar(j),ybar(j)
      write(12,*) "e "
      if(ilast.EQ.1) then
      close(12)
      endif
      return
      end
#endif
