! |$Id: bhod.F90,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
! --------------------------------------------------------------
! | Basic Seismic Utilities (BSU)                              |
! | LANGUAGE: Fortran77                                        |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: bhod.f                                       |
! | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
! | REVISION: Double precision version PLPLOT                  |
! |------------------------------------------------------------|
! | Determine tool orientation from PCA of hodogram            |
! | Note on Sign Convention for angles:                        |
! | Azimuth + clockwise from north=0deg Az                     |
! | Tan function Fortran, measured from east, + counter clockwz|
! |------------------------------------------------------------|
! |  NOTICE:                                                   |
! |  Copyright (C) 2017 Paul Michaels                          |
! |  <pm@cgiss.boisestate.edu>                                 |
! |  This program is free software; you can                    |
! |  redistribute it and/or modify it under the terms          |
! |  of the GNU General Public License as published            |
! |  by the Free Software Foundation; either version           |
! |  3 of the License, or (at your option) any later           |
! |  version.  This program is distributed in the              |
! |  hope that it will be useful, but                          |
! |  WITHOUT ANY WARRANTY; without even the implied            |
! |  warranty of MERCHANTABILITY or FITNESS FOR A              |
! |  PARTICULAR PURPOSE.  See the GNU General Public           |
! |  License for more details.                                 |
! |  You should have received a copy of the GNU                |
! |  General Public License along with this program;           |
! |  if not, write to the Free Software Foundation,            |
! |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
! |------------------------------------------------------------|
! | LIBRARY REQUIRED:  PLPLOT                                  |
! |                                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |                                                            |
! |REQUIRES: LAPACK (for eigenvalue decomposition)             |
! |http://www.netlib.org/lapack/                               |
! |------------------------------------------------------------|
! |  PARAMETERS:                                               |
! |  ndim = maximum number of samples in a seismic trace       |
! |------------------------------------------------------------|
! |  INCLUDE FILE REQUIRED:                                    |
! |  ../include/bsegy.f90    (modified segy header definitions)|
! |                (no reel header, floating point data)       |
! |------------------------------------------------------------|
! |  SUBROUTINES CALLED:                                       |
! | In-line===============                                     |
! |     GETPRM  get input parameters from command line or      |
! |             interactive query.  Should be inline with      |
! |             main program, specific to application.         |
! |     LSTPRM  echo check of input parameters from GETPRM.    |
! |             Values written to a history file (*.lst)       |
! |     HODPLT  Plots hodograms for QC. Requires PLPLOT package|
! |     FNDANG  Converts PCA vectors to azimuth angles based   |
! |             on a control target vector which resolves      |
! |             180 degree ambiguity (need to observe tool exit|
! |             from borehole in the field)                    |
! |     CORMAT  computes covariance matrix                     |
! |     OUTNIT  prints a matrix to listing file                |
! |     EIGEN   computes eigenvalues and eigenvectors          |
! |             Uses LAPACK: call ssyev()                      |
! |     SORT    sorts eigenvalues, drags eigenvectors along    |
! |             Sort is necessary, since major motion          |
! |             corresponds to larges eigenvalue, hence that   |
! |             eigenvector corresponds to R-component of my   |
! |             down-hole phone (may be different element on   |
! |             your phone, depends on wiring)                 |
! |                                                            |
! | Located in bsu-2.0.2/src/sublibF4===========               |
! |     CHKTRC  get number of samples, sample interval, and    |
! |             number of traces in the input data file.       |
! |             Also returns values for progress bar, stdout   |
! |     BSEGIN  Reads a seismic trace.                         |
! |     BSEGOUT Write a seismic trace.                         |
! |     PLTBAR  Writes progress bar on stdout                  |
! |                                                            |
! --------------------------------------------------------------
!
!
!
      include '../include/bsegy.f90'
      parameter (ndim=32767)
      parameter (ia=2)
!
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, hodlst
      byte              hdr(240),hdt(240)
      character*80      card
      integer*1         bz
      character*1024    cmd
!
      common /header/ hd
!
!////////////////\\\\\\ svd \\\\\\\\\\\/////////////////////////
!     integer p
      real*4    a(ia,ia),eigen(ia)
!///////////////////\\\\\\\\ pca \\\\\\\///////////////////////
      real*4 w(ia*2)
      real*4 xsia(ndim*2)
      real*4 xsibar(ndim*2)
!
!...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
!
!...define application prefix
      write(aplc,'(''bhod'')')
!
!...get parameters-------------------------------------------
      call GETPRM(nargsx,infil, &
      jchr,jcht,ipct,saz,azctl,tsw1)
!
!...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil, &
      bar,invbar,ibar,ntrace,ndim)
!
!...name outputfile
!      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
      write(hodlst,'(''bhod.lst'')')
!
!...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil, &
      infil,jchr,jcht,ipct,saz,azctl,tsw1)
!
!...clear output file...
!      open(9,file=outfil,access='direct',recl=240)
!      close(9,status='delete')
!
      icall1=0
      icall2=0
!
!...data loop=======================================....
!
!...read R-component
      jrec=jchr
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!...save header
      do 12 j=1,240
   12 hdr(j)=hd(j)
!
!      ntr = ntr + 1
!
!...read T-component
      jrec=jcht
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s2,fsamin,infil,jrec,iexit)
!...save header
      do 14 j=1,240
   14 hdt(j)=hd(j)
!      ntr = ntr + 1
!
!...construct motion 2-component vectors, xsia
      amaxm=-999999.
      do 50 jt=1,npts
      amp=(s1(jt)**2+s2(jt)**2)
      if(amp.ne.0.0) then
      amp=sqrt(amp)
      endif
      if(amp.gt.amaxm) then
      amaxm=amp
      jmaxm=jt
      endif
  50  continue
!
!...compute passing threshold
      thresh=(float(ipct)*amaxm)/100.
!
!
!...resample the trace for vectors above thresh
      kt=0
      do 51 jt=1,npts
      amp=(s1(jt)**2+s2(jt)**2)
      if(amp.ne.0.0) then
      amp=sqrt(amp)
      endif
      if(amp.ge.thresh) then
      kt=kt+1
      index=(kt-1)*2
      xsia(index+1)=s1(jt)
      xsia(index+2)=s2(jt)
      endif
  51  continue
!
!...compute principal components
      call svdpca(w,xsia,kt,2,2,2,io11, &
      a,eigen,xsibar)
!
!...compute azimuth for each component
      write(io11,FMT='('' '')')
      write(io11,FMT='('' '')')
      write(io11,FMT='(50(1h-))')
      write(io11,400) ipct,thresh,kt
  400 Format(2x,"Threshold=",i3,"%"/&
      2x,"Threshold Amplitude=",E14.7/&
      2x,"Number of Cases Above Threshold=",i5)
      tjmaxm=fsamin*float(jmaxm)
      write(io11,405) jmaxm,tjmaxm,amaxm
  405 Format(2x,"Maximum Amplitude at sample=",i5/ &
      2x,"Corresponding to sample time=",f10.3,"sec"/ &
      2x,"Maximum Amplitude=",E14.7)
!
!...convert angles to azimuth
      call fndang(w,saz,azr,azt,azctl,tsw1,io11)
!
!...open file to append with current tool orientation
      io15=15
      open(io15,file=hodlst,access='sequential',form='formatted')
!
!...move to end for appending to file
      krec=0
  600 continue
      read(UNIT=io15,FMT='(a)',END=601)   card
      krec=krec+1
      goto 600
  601 continue
      backspace(UNIT=io15)
!
!...seg-2 data (EGG) fixup for file numbers (ofrn=1001 instead of 1 etc.)
!      if(ofrn.gt.999)  ofrn=ofrn-3000
!
!...output R and T azimuths to a listing file
      write(io15,'(1x,i5.5,1x,f6.1,1x,f6.1)') ofrn,azr,azt
      close(UNIT=io15)
!
!...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
!
!
!...END LOOP========================================....
!
!...make postscript plot
      k=0
      do 101, i=1,kt
      k=k+1
      index=(i-1)*2
      S1(k)=xsia(index+1)
      S2(k)=xsia(index+2)
 101  continue
#ifdef PLPLOT
      call  hodplt(kt,S1,S2,azr,azt,ofrn,jchr,jcht,infil,ndim)
#else
      call  ghodplt(kt,S1,S2,azr,azt,ofrn,jchr,jcht)
      bz=0
      write(cmd,'(''gnuplot bhod.gp'',a)') bz
      call runcmd(cmd)

#endif
!...display progress....
      call pltbar(bar,invbar,ibar,ntrace,ntrace)
!
!
!...get time and date....
      write(*,'('' '')')
      write(6,240) ntrace,hodlst,outlst
240   format(/' Traces Processed = ',i5/ &
      ' OUTPUT FILE: ',a/' LISTING FILE: ',a)
#ifdef WIN
      write(outfil,'("h",i4.4,"plt.jpeg")') ofrn
#else
      write(outfil,'("h",i4.4,"plt.ps")') ofrn
#endif

      write(6,'(" IMAGE FILE: ",a13)') outfil
#ifndef PLPLOT
      write(6,*) "GNUPLOT FILE: bhod.gp  "
#endif
!
!
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: getprm.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! | get input parameters or help                               |
! --------------------------------------------------------------
!

      subroutine GETPRM(nargsx,infil, &
      jchr,jcht,ipct,saz,azctl,tsw1)
!
      character*40      infil
      character*40      arg1,arg2,arg3,arg4
      character*40      arg5,arg6,arg7
!
!
!...get input/output file names...
!   |--------------------------------------------|
!   | file naming convention                     |
!   | application prefix 4 characters "bxxx"     |
!   | data set suffix 4 characters "dddd"        |
!   | file name = bxxxdddd.seg                   |
!   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
!
!cc---LINUX VERSION
      nargsx=iargc()
      karg=-1
!...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bhod: Hodogram analysis by Principal     |'
      write(*,*) '| Components (PCA) (determines orientation)|'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*) &
      'bhod   infile chR chT ipct saz azctl tsw1'
      write(*,*) ' '
      write(*,*) 'infile =input file name '
      write(*,*) 'chR    = channel with R-component (int) '
      write(*,*) 'chT    = channel with T-component (int) '
      write(*,*) 'ipct   = percent of max amplitude to include (int) '
      write(*,*) 'saz    = source azimuth (ie E-W, then 90 deg) '
      write(*,*) 'azctl  = desired direction for R- (bowspring) '
      write(*,*) '       (azctl resolves 180 deg ambiguity PCA) '
      write(*,*) 'tsw1   = switch to set T-comp relative to R-comp '
      write(*,*) '       T-comp Azimuth= R-Comp + tsw1'
      write(*,*) '       Typically, tsw1= +90.(downhole) or -90.(ref)'
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
!
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter: chR (int) Channel with R-component'
      read(*,*) jchr
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') jchr
      endif
!
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  enter: chT (int) Channel with T-component'
      read(*,*) jcht
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') jcht
      endif
!
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  enter: ipct (percent of max amp to include, int)'
      read(*,*) ipct
      else
      call getarg(4,arg4)
      read(arg4,'(i5)') ipct
      endif
!
      if (nargsx.lt.(6+karg)) then
      write(*,*)'  enter: saz (source azimuth, ie E-W then 90deg)'
      read(*,*) saz
      else
      call getarg(5,arg5)
      read(arg5,'(f6.1)') saz
      endif
!
      if(nargsx.lt.(7+karg)) then
      write(*,*)'  enter: azctl (desired azimuth R- bowspring)'
      read(*,*) azctl
      else
      call getarg(6,arg6)
      read(arg6,'(f6.1)') azctl
      endif
!
      if (nargsx.lt.(8+karg)) then
      write(*,*)'  enter: tsw1 (relative azimuth of T- wrt R-)'
      write(*,*)'  T-azimuth=R-azimuth + tsw1'
      read(*,*) tsw1
      else
      call getarg(7,arg7)
      read(arg7,'(f6.1)') tsw1
      endif
!
      return
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: lstprm.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! | echo check of input parameters, writes to a history file   |
! --------------------------------------------------------------
!
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil, &
      infil,jchr,jcht,ipct,saz,azctl,tsw1)
      character*4       aplc
      character*40      outlst,outfil,infil
!
!...open output listing file...
      io11=10
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2000  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|---------------------------------|'
      write(io11,'('' Program bhod  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   R-Channel= '',i5)') jchr
      write(io11,'(''   T-Channel= '',i5)') jcht
      write(io11,'(''       ipct = '',i5)') ipct
      write(io11,'(''       saz  = '',f6.1)') saz
      write(io11,'(''      azctl = '',f6.1)') azctl
      write(io11,'(''      tsw1  = '',f6.1)') tsw1
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
!
      return
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: fndang.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! | Converts PCA vectors to azimuth angles based on source az. |
! | Then resolves ambiguity of 180 degrees.                    |
! |------------------------------------------------------------|
! |                                                            |
! | Note on Sign Convention for angles:                        |
! | Azimuth + clockwise from north=0deg Az                     |
! | Tan function Fortran, measured from east, + counter clockwz|
! |------------------------------------------------------------|
! | saz=source azimuth (ex:if source east-west, saz=90 )       |
! | azr=azimuth of R-component  azt=azimuth of T-component     |
! |azctl=azimuth in desired direction to resolve ambiguity     |
! | Thus, if angle abs(azr-vect)>90 degrees,    add 180 to azr |
! |------------------------------------------------------------|
! | tsw1=+90 or -90 degrees depending on following:            |
! | The down hole phone is wired so that the T-comp has an     |
! | azimuth 90deg more than the R-comp.                        |
! |                                                            |
! | The reference phone is wired so that the T-comp has an     |
! | azimuth 90 deg less than the R-comp.                       |
! |------------------------------------------------------------|
      subroutine fndang(w,saz,azr,azt,azctl,tsw1,io11)
      real*4 w(*)
!
!...convert angles to azimuth
      azr=saz-atan2(w(2),w(1))*(180/3.1415926)
      azt=saz-atan2(w(4),w(3))*(180/3.1415926)
      write(io11,FMT='('' '')')
      write(io11,FMT='('' '')')
      write(io11,FMT='(50(1h-))')
      write(io11,FMT='(''AZIMUTHS of COMPONENTS'')')
      write(io11,FMT='(5x,''AZ-R='',f7.2,''Deg.'')') azr
      write(io11,FMT='(5x,''AZ-T='',f7.2,''Deg.'')') azt
      write(io11,FMT='(50(1h-))')
      write(io11,FMT='('' '')')
      write(io11,FMT='('' '')')
      write(io11,FMT='(''====Adjusted Orthogonal Estimates===='')')
!
!...make sure azimuth angles positive 0<=az<=360
      azr=azr+360.
      if(azr.gt.360.) azr=azr-360.
      azt=azr+tsw1
      if(azt.gt.360.) azt=azt-360.
      if(azt.lt.0.) azt=azt+360.
!
!...compare azr to azctl, and resolve ambiguity
!--------begin fix-------------------------------
!...bug fix 23 May 2004  Use Innerproduct to determine distance from
!   target direction azctl (problem near north) P.M.
      cv=3.1415926/180.
      xr=cos(azr*cv)
      yr=sin(azr*cv)
      xtarg=cos(azctl*cv)
      ytarg=sin(azctl*cv)
      fipr=xr*xtarg+yr*ytarg
      adist=acos(fipr)/cv
      if (adist.gt.90.0) azr=azr+180.0
!----------------------------------------end of fix
      if(azr.gt.360.) azr=azr-360.
      azt=azr+tsw1
      if(azt.gt.360.) azt=azt-360.
      if(azt.lt.0.) azt=azt+360.
      write(io11,FMT='(5x,''AZ-R='',f7.2,''Deg.'')') azr
      write(io11,FMT='(5x,''AZ-T='',f7.2,''Deg.'')') azt
      return
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: svdpca.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | svdpca: principal component analysis by eigenvalue         |
! |         decomposition of correlation matrix                |
! |                                                            |
! |  xsi =  input vector   (m x 1)                             |
! |  f   =  output vector  (n x 1)                             |
! |                                                            |
! |  w   =  weight matrix  (n x m) (each row is an eigenvector |
! |                            of the input correlation matrix)|
! |  ndim = dimension of output vector = n                     |
! |  mdim = dimension of input vector  = m                     |
! |                                                            |
! |  <<<<<<<<<<<<<<< version for use with bhod.f >>>>>>>>>>>>  |
! |------------------------------------------------------------|
! |  subroutines used:                                         |
! |   eigen.f  sort.f cormat.f outnit.f                        |
! |------------------------------------------------------------|
!
      subroutine svdpca(w,xsia,nvec,mdim,ndim,ia,io11, &
      a,eigenv,xsibar)
      implicit real*4 (a-h,o-z)
!////////////////\\\\\\ svd \\\\\\\\\\\/////////////////////////
      real*4    a(ia,ia),eigenv(ia)
!///////////////////\\\\\\\\ pca \\\\\\\///////////////////////
      real*4 w(*)
      real*4 xsia(*)
      real*4 xsibar(*)
!
!
!  <<<<<<<<<<<<<<< begin input block>>>>>>>>>>>>>>>>>>
!...read data
      if(ndim.gt.ia) then
      write(*,*) 'abort--ndim too large'
      stop
      endif
      if(mdim.gt.ia) then
      write(*,*) 'abort--mdim too large'
      stop
      endif
!
      do 5 k=1,mdim
    5 xsibar(k)=0.0
!
      do 10 j=1,nvec
      do 12 k=1,mdim
      index=(j-1)*mdim+k
!...compute average input vector
      xsibar(k)=xsibar(k)+xsia(index)/float(nvec)
   12 continue
   10 continue
!
!  <<<<<<<<<<<<<<<end of input block>>>>>>>>>>>>>>>>>>>
!
      write(io11,'(''  average input vector:''/)')
      write(io11,'(5(1x,e14.7))') (xsibar(k),k=1,mdim)
!
!...remove mean from input vectors
      do 120 j=1,nvec
      do 100 k=1,mdim
      index=(j-1)*mdim+k
      xsia(index)=xsia(index)-xsibar(k)
  100 continue
  120 continue
!
!...compute <correlation matrix>
      call cormat(nvec,mdim,xsia,a,ia)
!
!...set dimensions
      n=mdim
!
!...perform eigenvalue decomposition
      write(io11,'(//''  Correlation Matrix'')')
      call outnit(a,n,n,ia)
!
      call eigen(a,n,eigenv,ia)
!
      write(io11,'(//''  Eigenvalues'')')
      call outnit(eigenv,n,1,ia)
!
      write(io11,'(//''  Eigenvectors '')')
      call outnit(a,n,n,ia)
!
!...copy eigenvectors into weight matrix, w
      do 75 jx=1,ndim
      indxj=(jx-1)*mdim
      do 75 kx=1,mdim
      w(indxj+kx)=a(kx,jx)
   75 continue
!
!...final  ////////////////////////////
!...restore mean to vectors
      do 130 j=1,nvec
      do 130 k=1,mdim
      index=(j-1)*mdim+k
      xsia(index)=xsia(index)+xsibar(k)
  130 continue
!
!
      write(io11,'(//''  weights: Row Vectors (R,T)''/)')
!     write(*,'(//''  weights:''/)')
      do 62 i=1,ndim
      index=(i-1)*mdim
      write(io11,901) (w(index+kk),kk=1,mdim)
!     write(*,901) (w(index+kk),kk=1,mdim)
  901 format(5(2x,e10.4))
   62 continue
!
      return
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: cormat.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | cormat:  computes correlation matrix summed over           |
! |          number of exemplars                               |
! |------------------------------------------------------------|
      subroutine cormat(nvec,mdim,xsia,matrix,ia)
      implicit real*4 (a-h,o-z)
      real*4 xsia(*),matrix(ia,ia)
!
!...zero correlation matrix
      do 10 j=1,mdim
      do 10 k=1,mdim
   10 matrix(j,k)=0.00
!
!...loop over number of vectors
      do 100 jv=1,nvec
!
      index=(jv-1)*mdim
      do 50 j=1,mdim
      indxj=index+j
      do 50 k=1,mdim
      indxk=index+k
   50 matrix(j,k)=matrix(j,k) + xsia(indxj)*xsia(indxk)
!
  100 continue
      return
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: eigen.f                                      |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! | eigen.f      find eigenvalues and eigenvectors for symetric|
! |              matrix using LAPACK routines                  |
! |------------------------------------------------------------|
! |  inputs:                                                   |
! |  a= matrix (n x n) input symetric matrix                   |
! |  ia=physical dimension                                     |
! |  n=logical dimension                                       |
! |                                                            |
! |------------------------------------------------------------|
! |  outputs:                                                  |
! |           w=eigenvalues                                    |
! |           a=matrix of eigenvectors (columns)               |
! |------------------------------------------------------------|
      subroutine eigen(a,n,w,ia)
      parameter (ldim=1000)
      dimension a(ia,*),w(*)
      character*1  jobz,uplo
      real work(ldim)
      integer info
!
!...define some parameters
!     jobz=v compute eigenvalues and eigenvectors
!     jobz=N compute eigenvalues only
      jobz='V'
      uplo='U'
      lda=n
      lwork=3*n-1
      if(lwork.gt.LDIM) then
      write(*,*) 'ABORT-work dimension exceeded in eigen.f'
      stop
      endif
!
      call ssyev(jobz,uplo,n,a,lda,w,work,lwork,info)
      call sort(w,a,n,ia)
!
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: sort.f                                       |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | sort: sorts eigenvalues and eigenvectors simultaneously    |
! |       a(1)=maximum  a(n)=min                               |
! |------------------------------------------------------------|
! |  inputs:                                                   |
! |  a= vector of eigenvalues                                  |
! |  u= matrix of eigenvectors                                 |
! |                                                            |
! |------------------------------------------------------------|
! |  outputs:                                                  |
! |  a= sorted                                                 |
! |  u= sorted                                                 |
! |                                                            |
! |------------------------------------------------------------|
      subroutine sort(a,u,n,ia)
      dimension a(ia),u(ia,ia)
      lim=n-1
    1 int=n
      do 5 i=1,lim
      k=n+1-i
      j=k-1
      if (a(k)-a(j)) 5,5,6
!...interchange if a(k).gt.a(j)
    6 temp=a(j)
      a(j)=a(k)
      a(k)=temp
      int=k
      do 8 l=1,n
      hold1=u(l,j)
      u(l,j)=u(l,k)
    8 u(l,k)=hold1
    5 continue
      if(int.eq.n) return
      go to 1
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: outnit.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | output: outputs matrix                                     |
! |                                                            |
! |------------------------------------------------------------|
! |  inputs:                                                   |
! |  a= matrix to list                                         |
! |  dimensions a(n,m) logical                                 |
! |             a(ia,ia) physical                              |
! |                                                            |
! |------------------------------------------------------------|
! |  outputs:                                                  |
! |  list of matrix to file, unit 10                           |
! |                                                            |
! |                                                            |
! |------------------------------------------------------------|
      subroutine outnit(a,n,m,ia)
      real a(ia,ia)
      kkt=m/5
      kt=5
      kb=1
      if (m.lt.5) go to 71
      do 70 kk=1,kkt
      do 60 j=1,n
!     write(*,218) j,(a(j,k),k=kb,kt)
   60 write(10,218) j,(a(j,k),k=kb,kt)
      write(10,209)
      write(10,209)
!     write(*,209)
!     write(*,209)
      kb=kt+1
   70 kt=kt+ 5
   71 continue
      do 80 j=1,n
!     write(*,218) j,(a(j,k),k=kb,m)
   80 write(10,218) j,(a(j,k),k=kb,m)
  209 format(1x)
  218 format(/1x,i3,5(2x,e11.5))
      return
      end
#ifdef PLPLOT
!#
! --------------------------------------------------------------
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | subroutine: hodplt.f                                       |
! |                                                            |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! | Plots a hodogram as points                                 |
! |                                                            |
! | Note on Sign Convention for angles:                        |
! | Azimuth + clockwise from north=0deg Az                     |
! | Tan function Fortran, measured from east, + counter clockwz|
! |REQUIRES PLPLOT:                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |                                                            |
! --------------------------------------------------------------
      subroutine hodplt(npts,r,t,azr,azt,ofrn,jchr,jcht,infil,ndim)
      use plplot
      real*4 r(npts),t(npts)
      real*8 xd(npts),yd(npts)
      real*8 x(2),y(2)
      real*8 fdef,fscale,fzero
      real*8 fmargin,aspect,yjust,xjust
      real*8 xmn,xmx,ymn,ymx
      real*8 amaxm
      real*8 fwidth
!
      integer*4 ofrn
      character*14 pltfil
      character*40 titl
      character*40 xlbl
      character*40 ylbl
      character*40 infil
!
!...open device
!  Portrait
      izero=0
      ione=1
!...copy to double precision
      do 102, i=1,npts
      xd(i)=r(i)
      yd(i)=t(i)
102   continue
      call plscolor(izero)
      call plsori(ione)
!...aspect ratio needed for portrait, unless you don't mind
!   plots which are squished in the vertical dimension.
!   aspect=8.5/11.0
          fmargin=0.
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('ps')
          write(pltfil,'(a,i4.4,''plt.ps'')') infil(1:1),ofrn
!
      call plsdidev(fmargin,aspect,yjust,xjust)
!
      call plsfnam(pltfil)
!
!...initialize plot
      nx=1
      ny=1
      call plinit()
      call plssub(nx,ny)
      call plfontld(izero)
!
!...determine maximum and minimums
      fmax=r(1)**2+t(1)**2
      if (fmax.ne.0.0) fmax=sqrt(fmax)
      fmin=fmax
      do 10, i=2,npts
      tst=r(i)**2+t(i)**2
      if(tst.ne.0.0) tst=sqrt(tst)
      fmax=max(tst,fmax)
      fmin=min(tst,fmin)
   10 continue
      amaxm=max(abs(fmax),abs(fmin))
!
!...set environment, window and viewport
!      call plenv(-amaxm,amaxm,-amaxm,amaxm,0,0)
      izero=0
      ione=1
      iwidth=1
      fwidth=1.0
      fdef=0.
      fscale=0.5
      call pladv(izero)
      call plschr(fdef,fscale)
      call plprec(izero,izero)
      call plssym(fdef,fscale)
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      xmn=0.1
      xmx=.55
      ymn=0.6
      ymx=.95
      call plvpor(xmn,xmx,ymn,ymx)
      call plwind(-amaxm,amaxm,-amaxm,amaxm)
      fzero=0.
      izero=0
      call plbox('abcnst',fzero,izero,'abcnstv',fzero,izero)
!
      write(titl,'(''Hodogram: Shot='',i4.4,'' R='',f6.1, &
      '' T='',f6.1)') ofrn,azr,azt
      write(xlbl,'(''R-Comp. ch='',i2.2)') jchr
      write(ylbl,'(''T-Comp. ch='',i2.2)') jcht
      call pllab(xlbl,ylbl,titl)
      icode=9
!      call plpoin(npts,r,t,icode)
      call plpoin(xd,yd,icode)
!
!...plot lines along determined azimuths
      x(1)=0.
      y(1)=0.
      angl=3.1415926/180.0
      x(2)=(amaxm/2.)*cos((90.0-azr)*angl)
      y(2)=(amaxm/2.)*sin((90.0-azr)*angl)
      call plptex(x(2),y(2),fzero,fzero,fzero,'R')
      call plline(x,y)
      x(2)=(amaxm/2.)*cos((90.0-azt)*angl)
      y(2)=(amaxm/2.)*sin((90.0-azt)*angl)
      call plptex(x(2),y(2),fzero,fzero,fzero,'T')
      call plline(x,y)
      call plend()
      return
      end

#else
!#
! --------------------------------------------------------------
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | subroutine: ghodplt.f                                      |
! |                                                            |
! | copyright (c) 2017 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! | Plots a hodogram as points                                 |
! |                                                            |
! | Note on Sign Convention for angles:                        |
! | Azimuth + clockwise from north=0deg Az                     |
! | Tan function Fortran, measured from east, + counter clockwz|
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |                                                            |
! --------------------------------------------------------------
      subroutine ghodplt(npts,r,t,azr,azt,ofrn,jchr,jcht)
      real*4 r(npts),t(npts)
      real*4 x(2),y(2)
      real*4 Tx,Ty,Rx,Ry
      integer*4 ofrn
      character*80 sxrange,syrange
      character*20 outfile
! fill outfile name
! Note: Microsoft Windows does not work well with Postscript
! so compile for a jpeg output.
#ifdef WIN
      write(outfile,'("h",i4.4,"plt.jpeg")') ofrn
#else
      write(outfile,'("h",i4.4,"plt.ps")') ofrn
#endif
!
!...determine maximum and minimums
      fmax=r(1)**2+t(1)**2
      if (fmax.ne.0.0) fmax=sqrt(fmax)
      fmin=fmax
      do 10, i=2,npts
      tst=r(i)**2+t(i)**2
      if(tst.ne.0.0) tst=sqrt(tst)
      fmax=max(tst,fmax)
      fmin=min(tst,fmin)
   10 continue
      amaxm=max(abs(fmax),abs(fmin))
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")')-amaxm,amaxm 
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")')-amaxm,amaxm
! compute locations for labels
      angl=3.1415926/180.0
      Rx=(amaxm/2.)*cos((90.0-azr)*angl)
      Ry=(amaxm/2.)*sin((90.0-azr)*angl)
      Tx=(amaxm/2.)*cos((90.0-azt)*angl)
      Ty=(amaxm/2.)*sin((90.0-azt)*angl)
      open(unit=12,file='bhod.gp',access='sequential')
      write(12,*) "# Generated by bhod.F90  Subroutine ghodplt()     "
#ifdef WIN
      write(12,*) "set terminal jpeg large size 500,500 enhanced"
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a13,"'")
#else
      write(12,*) "set terminal postscript size 5.0,5.0 in \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a11,"'")
#endif 
      write(12,*) "set grid                                          "
      write(12,*) "set nokey" 
      write(12,*) "set xlabel '\"                                    
      write(12,'("R-Component, ch=",i2.2," \")') jchr
      write(12,*) " ' "
      write(12,*) "set ylabel '\"
      write(12,'("T-Component, ch=",i2.2," \")') jcht
      write(12,*) " ' "
      write(12,*) "set title 'Hodogram: Shot=\"
      write(12,'(i4.4," R=",f6.1," T=",f6.1,"\")') ofrn,azr,azt
      write(12,*) " '"
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,110) Rx,Ry
  110 format("set label 'R ' at ",E14.7," , ",E14.7," right")
      write(12,120) Tx,Ty
  120 format("set label 'T ' at ",E14.7," , ",E14.7," right")
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) w p pt 6 ps 1, \"
      write(12,*) " '' u ($1):($2) w l ls 1  , \"
      write(12,*) " '' u ($1):($2) w l ls 1 "
      do 20, j=1,npts
   20 write(12,'(2(E14.7,1x))') r(j),t(j)
      write(12,*) "e "
!
!...plot lines along determined azimuths
      x(1)=0.
      y(1)=0.
      x(2)=Rx
      y(2)=Ry
      do 30, j=1,2
   30 write(12,'(2(E14.7,1x))') x(j),y(j)
      write(12,*) "e "
      x(2)=Tx
      y(2)=Ty
      do 40, j=1,2
   40 write(12,'(2(E14.7,1x))') x(j),y(j)
      write(12,*) "e "
      close(12)
      return
      end

#endif
