c |$Id: brsp.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: brsp.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |   interpolation (resampling) by augmentation with zeros    |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     NLOGN   FFT from E.A. Robinson (radix 2)               |
c |     NRAD2   Finds number samples appropriate for radix 2   |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      complex           x(ndim)
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''brsp'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +ifact,tmax1)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,ifact,tmax1)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...take fft of trace
      nptsx=tmax1/fsamin+1
      call nrad2(nptsx,npr2)
      npts2=npr2*(2**ifact)
      if(npts2.gt.ndim) then
      write(*,*) 'ABORT: Too many samples will result'
      write(*,*) 'RADIX 2 Algorithm  '
      write(*,'(''npts='',i10)') npts
      write(*,'(''With tmax, nptsx='',i10)') nptsx
      write(*,'(''With Radix2, npr2='',i10)') npr2
      write(*,'(''After Interpolation, npts2='',i10)') npts2
      write(*,'(''Max Dimension is'',i10)') ndim
      stop
      endif
      do 20 jt=1,nptsx
   20 x(jt)=cmplx(s1(jt),0)
      do 30 jt=(nptsx+1),npr2
   30 x(jt)=cmplx(0.,0.)
      sign=-1.0
      call nlogn(npr2,x,sign)
c
c...shift half of spectrum and augment with zeros in middle
      jstop=npr2/2+1
      do 40 jt=2,jstop
   40 x(npts2-jt+2)=x(npr2-jt+2)
      do 50 jt=(jstop+1),(npts2-jstop+1)
   50 x(jt)=cmplx(0.,0.)
c
c...take inverse fft
      sign=+1.0
      call nlogn(npts2,x,sign)
c
c...extract signal
      nptss=nptsx*(2**ifact)
      do 70 jt=1,nptss
   70 s2(jt)=real(x(jt))
c
c...output trace
      fsamin2=fsamin/(2**ifact)
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,nptss,s2,fsamin2,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c |                                                            |
c | getprm.for   B-SEGY format (no reel header, floating point)|
c |------------------------------------------------------------|
c | get parameters and help                                    |
c |                                                            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +ifact,tmax1)
c
      character*40      infil
      character*40      arg1,arg2
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| brsp: Resample data by interpolation     |'
      write(*,*) '| (augment with zeros in frequency domain) |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'brsp infile ifact tmax'
      write(*,*)' '
      write(*,*)'infile  =input file name '
      write(*,*)'ifact   = resample factor ifact>=1 '
      write(*,*)'        = 1 sample interval halved.  '
      write(*,*)'        = 2 sample interval 1/4 of original'
      write(*,*)'        = n sample interval 1/(2**n) of original'
      write(*,*)' '
      write(*,*)'    (note: trace size increases by 2**ifact)'
      write(*,*)'tmax    =max time of output trace (float) sec' 
      write(*,*)' '
      write(*,*)'NOTE: This is a radix 2 algorith, so trace may be '
      write(*,*)' with zeros before computing new number of samples '
      write(*,*)'   BSEGY number of samples header is 16 bit'
      write(*,*)'   Maximum number of samples limited to 32,767'
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter: ifact'
      write(*,*)'    (note: trace size increases by 2**ifact)'
      read(*,*) ifact
      else
      call getarg(2,arg2)
      read(arg2,'(i1)') ifact 
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  enter: tmax'
      read(*,*) tmax1 
      else
      call getarg(3,arg2)
      read(arg2,'(f3.4)') tmax1 
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c |                                                            |
c | LSTPRM.for   B-SEGY format (no reel header, floating point)|
c |------------------------------------------------------------|
c | echo echeck                                                |
c |                                                            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,ifact,tmax1)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program brst  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   ifact= '',i2)') ifact 
      write(io11,'(''   tmax= '',f10.4)') tmax1
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
