! |$Id: bvas.F90,v 1.3 2017/10/22 23:37:53 pm Exp $            |
! --------------------------------------------------------------
! | Basic Seismic Utilities (BSU)                              |
! | LANGUAGE: Fortran77                                        |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: bvas.f                                       |
! | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
! | REVISION: Double precision version PLPLOT                  |
! |------------------------------------------------------------|
! | Velocity analysis program for Vertical Seismic Profiles    |
! | Computes velocity dispersion (as a function of frequency)  |
! | by a semblance technique.  Assuming VERY narrow band-pass  |
! | filters are selected, the velocities are essentially       |
! | phase velocities.  The frequency increment may be larger   |
! | than the bandwidth of the filters, (set by                 |
! | the data aperture (1/(npts*fsamin)). The results are       |
! | written to a history file.  Postscript plots are also      |
! | generated with the plplot library (required for plotting). |
! | The bvas.his history file is used with the bamp.his file   |
! | To perform a joint inversion for the two wave equation     |
! | coefficients (stiffness and damping) for the 3rd order PDE |
! | consistent with a Kelvin constitutive model.               |
! |------------------------------------------------------------|
! |  NOTICE:                                                   |
! |  Copyright (C) 2017 Paul Michaels                          |
! |  <pm@cgiss.boisestate.edu>                                 |
! |  This program is free software; you can                    |
! |  redistribute it and/or modify it under the terms          |
! |  of the GNU General Public License as published            |
! |  by the Free Software Foundation; either version           |
! |  3 of the License, or (at your option) any later           |
! |  version.  This program is distributed in the              |
! |  hope that it will be useful, but                          |
! |  WITHOUT ANY WARRANTY; without even the implied            |
! |  warranty of MERCHANTABILITY or FITNESS FOR A              |
! |  PARTICULAR PURPOSE.  See the GNU General Public           |
! |  License for more details.                                 |
! |  You should have received a copy of the GNU                |
! |  General Public License along with this program;           |
! |  if not, write to the Free Software Foundation,            |
! |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
! |------------------------------------------------------------|
! |  PARAMETERS:                                               |
! |  ndim = maximum number of samples in a seismic trace       |
! |------------------------------------------------------------|
! |  INCLUDE FILE REQUIRED:                                    |
! |  ../include/bsegy.inc    (modified segy header definitions)|
! |                (no reel header, floating point data)       |
! |------------------------------------------------------------|
! |  LIBRARY REQUIRED:   (Double Precision PLPLOT)             |
! |                                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |------------------------------------------------------------|
! |  SUBROUTINES and FUNCTIONS CALLED:                         |
! | In-line===============                                     |
! |     GETPRM  get input parameters from command line or      |
! |             interactive query.  Should be inline with      |
! |             main program, specific to application.         |
! |     LSTPRM  echo check of input parameters from GETPRM.    |
! |             Values written to a history file (*.lst)       |
! |     STAGE1  Computes geometric information                 |
! |     ZRANGE  Selects a subset of data for analysis by elev. |
! |     FILTER  Filters traces with band-pass                  |
! |     VELSCN  Semblance scan of velocities                   |
! |     VPLOT2  Velocity dispersion, generates Postscript plot |
! |     VPLOT   QC Postscript plot at each frequency           |
! |             (cross-correl. time shifts about correction    |
! |              velocity; good is tight pattern about a       |
! |              horizontal line)                              |
! |     VSHFT   Shifts traces for semblance calculation        |
! |     APLYV   Calls VSHFT, screens against horizontal.       |
! |     FSIGNAL Windows innerproduct to focus on good signal   |
! |     APLYV   Calls VSHFT, screens against horizontal.       |
! |     GS      Golden Section Search for MAXIMUM semblance    |
! |     FNEWX   Called by GS, Function to find next trial sol. |
! |     F       The function used by GS.f                      |
! |     VALXX   Interpolate a sample FUNCTION                  |
! |     TVAR    Computes a variance in time for error bars.    |
! |                                                            |
! | Located in bsu-2.0.2/src/sublibF4===========               |
! |     CHKTRC  get number of samples, sample interval, and    |
! |             number of traces in the input data file.       |
! |             Also returns values for progress bar, stdout   |
! |     BSEGIN  Reads a seismic trace.                         |
! |     BSEGOUT Write a seismic trace.                         |
! |     ARMA    Filter program                                 |
! |     FINDXYZ Gets header geometry                           |
! |     L2NORM  Scales a trace by L2 norm                      |
! |                                                            |
! --------------------------------------------------------------
!
      PROGRAM MAIN
      include '../include/bsegy.f90'
      parameter (ndim=32767)
      parameter (mdim=1000)
!
      real*4            depth(mdim),angle(mdim),cosa(mdim)
      real*4            relev(mdim)
      real*4            frq(mdim),cvel(mdim),semblx(mdim)
      real*4            hldvel(mdim),hldsmb(mdim),tshift(mdim)
      real*4            hldtbr(mdim),hldtvr(mdim),sigmam(mdim)
      character*80      card
!
      real*4            s1(ndim),s2(ndim),s4(ndim),s5(ndim)
!
      character*40      infil , outfil, aplc, outlst
      character*1024    cmd
      integer*1         bz
      common /header/ hd
!
!...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
!
!...July2012 update for frequency increments less than 1 hz
!   scale factor for file names to be in tenths of Hz
      SF=10.0
!
!...define application prefix
      write(aplc,'(''bvas'')')
!
!---------------START HARDWIRE OPTIONS------------------------
!...zerophase filters
      ifaz=0
!...minimum phase filters
!     ifaz=1
!
      itype=1
      if(ifaz.eq.1) then
      npoles=14
      else
      npoles=14
      endif
      npow=npoles/2
      npoles=2*npow
!----------------END HARDWIRE OPTIONS-------------------------
!
!...get parameters-------------------------------------------
      call GETPRM(nargsx,infil, &
      emin,emax,vmin,vmax,nvel,fmin,fmax,delf,bwidth,iskp,ivscn)
!
!...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil, &
      bar,invbar,ibar,ntrace,ndim)
!
!...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
!
!...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,ifaz,npoles, &
      emin,emax,vmin,vmax,nvel,fmin,fmax,delf,bwidth,iskp,ivscn, &
      aplc,outlst,outfil,infil)

!
!...STAGE 1: Load Geometry Tables----------------------------
      icall1=0
      call STAGE1(npts,ntrace,s1,depth,angle,cosa,io11,ndim, &
      relev,infil,icall1,iunit1,fsamin,mdim)
!
!...compute depth increment
      deltaz=depth(1)-depth(2)
      write(io11,'(''  Depth increment= '',f10.2)') deltaz
!
!...data loop=======================================....
!
!...filter setting
      nfilt=int((fmax-fmin)/delf+1)
!
!...trace equalization gate
      tmin1=0.
      tmax1=float(npts-1)*fsamin
!
!...depth of interest
      call ZRANGE(ntrace,emin,emax,jmin,jmax,depth,relev, &
      zmin,zmax,io11)
!
      ntrhld=0
      icall7=0
!
!...history file
      open(unit=3,file='bvas.his',access='sequential')
!
!...frequ, velocity, semblance file
      open(unit=15,file='semblance.dat',access='sequential') 
!
!...move to end of file
      krec=0
  101 continue
      read(3,'(a)',end=102)  card
      krec=krec+1
      go to 101
  102 continue
      rewind(3)
      do 103 jr=1,krec
  103 read(3,'(a)') card
!
!...filter loop-----------------------////
      ilast=0
      do 300 jfilt=1,nfilt
      fcenter=fmin+float(jfilt-1)*delf
      ifcntr=int(fcenter*SF)
      if(itype.ne.1) then
      bwidth=1.*fcenter
      endif
!
!...STAGE 2: Filter data-------------------------------------
      write(outfil,'(a,i5.5,''.seg'')') aplc(1:1),ifcntr
!
!...clear output file...
      if(iskp.le.0) then
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
      endif
!
      icall1=0
      icall2=0
!
!...need to close unit1 under gfortran.  g77 would permit redefinition
!   of recl on open file, gfortran does not.
      close(iunit1)
!
      if(iskp.le.0) then
      call FILTER(npts,ntrace,s1,infil,iunit1, &
      icall1,fsamin,s2,outfil,iunit2,icall2, &
      itype,bwidth,npoles,fcenter,tmin1,tmax1,ndim,ifaz)
      else
      jrec=1
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,outfil,jrec,iexit)
      if(iexit.eq.1) then
      write(*,*)' ABORT--Filter file missing'
      close(iunit1,status='delete')
      stop
      endif
      read(outfil,'(4x,i5)') ifcentr
      fcenter=float(ifcentr)/SF
      write(*,*)' '
      write(*,'(''  FREQUENCY= '',f8.1)') fcenter
      endif
!
      write(io11,'(50(1h-))')
      write(io11,'(''  FREQUENCY= '',f10.1)') fcenter
!
      close(iunit1)
      close(iunit2)
!
!...STAGE 3: Velocity scan-----------------------------------
      icall1=0
      xxx=abs(zmax-zmin)
!
      semblx(jfilt)=-1.0E+20
      icall7=0
      ntrhld=0
!
      jndex=1
      do 200 jscn=1,nvel
!...application velocity in equal time steps
      xtmin=xxx/vmax
      xdelt=(xxx/float(nvel-1))*(1.0/vmin-1.0/vmax)
      vel=xxx/(xtmin+float(nvel-jscn)*xdelt)
!
      call VELSCN(npts,s1,outfil,iunit1,fcenter, &
      icall1,fsamin,s2,depth,cosa,angle,ivscn,ndim, &
      vel,sembl,jmin,jmax,s4,s5,ntrhld,icall7,SF)
!
!...output semblance value
      write(io11,1101) vel,sembl
1101  format('  Velocity=',f8.1,' Semblance=',f10.4)
!
!...output to semblance.dat file 
      write(15,'(E14.7,1x,E14.7,1x,E14.7)') fcenter,vel,sembl
!
!...search for maximum semblance
      if(sembl.gt.semblx(jfilt)) then
      jndex=jscn
      semblx(jfilt)=sembl
      endif
!
!...save semblance and velocity
      hldsmb(jscn)=sembl
      hldvel(jscn)=vel
!
  200 continue
!
!...golden section search
!...check for bracket failure
      if(jndex.eq.1) then
      write(*,*)'  ABORT--vmin too large'
      endif
      if(jndex.eq.nvel) then
      write(*,*)'  ABORT--vmax too small'
      endif
      if(jndex.eq.1.or.jndex.eq.nvel) then
      frq(jfilt)=fcenter
      cvel(jfilt)=0.0
      sigmam(jfilt)=0.0
      hldtbr(jfilt)=0.0
      hldtvr(jfilt)=0.0
      semblx(jfilt)=0.0
!...clear for vplot
      do 711, kx=1,(jmax-jmin+1)
      tshift(kx)=0.0
      tbar=0.
      tvr=0.
  711 continue
      go to 888
      endif
!
!...initial bracket
      gax=hldvel(jndex-1)
      gbx=hldvel(jndex)
      gcx=hldvel(jndex+1)
!
      semblx(jfilt)=GS(gax,gbx,gcx,xsol, &
      npts,s1,infil,iunit1,icall1,fsamin,s2,depth,cosa, &
      angle,jmin,jmax,s4,s5,ncnt)
!
      cvel(jfilt)=xsol
      frq(jfilt)=fcenter
!
!...compute relative time shifts
      write(io11,1102)
1102  format(1x,57(1h_))
      tvr=tvar(xsol,npts,s1,infil,iunit1, &
      icall1,fsamin,s2,depth,cosa,angle,ndim,io11, &
      jmin,jmax,s4,tshift,fcenter,tbar)
!
      write(io11,900) cvel(jfilt),tbar,tvr
 900  format(//5x,'Velocity=',f10.3,' Average Shift (sec.)=',f10.5/ &
      25x,'Shift Variance (sec.)=',f10.5/)
      hldtbr(jfilt)=tbar
      hldtvr(jfilt)=tvr
      write(io11,*) '     Relative Time Shifts (sec.)'
      write(io11,*) 'Trace_no  Elev       Depth    Time_Shift'
      ntrcmx=jmax-jmin+1
!
!...output relative time shifts
      do 202, kkk=1,ntrcmx
      kkktrc=(kkk-1)+jmin
      write(io11,901) kkktrc,relev(kkktrc),depth(kkktrc), &
      tshift(kkk)

 202  continue
 901  format(2x,i5,2x,f8.2,2x,f8.2,2x,f10.5)
!
!...compute error bar
      sumx=0.
      sumx2=0.
      sumy=0.
      sumxy=0.
      kkk=0
      do 147 j1=jmin,jmax
      kkk=kkk+1
      sumx=sumx+depth(j1)
      sumx2=sumx2+depth(j1)*depth(j1)
      ty=(depth(j1)/(cvel(jfilt)))+(tshift(kkk)-tbar)
      sumy=sumy+ty
      sumxy=sumxy+ty*depth(j1)
  147 continue
      sumn=0.
      fltn=float(ntrcmx)
      temp1=fltn*(fltn*sumx2-sumx*sumx)**3
      temp2=(sumx*sumy-fltn*sumxy)**4
      fmult=tvr
      if(fmult.lt.fsamin) fmult=fsamin
      sigmam(jfilt)=fmult*(sqrt(temp1)/sqrt(temp2))
!
!...branch to here if bracket failure
!   cvel(jfilt)=0 indicates bracket failure in bvas.his
  888 continue
!
!...generate postscript plot
      if(jfilt.eq.nfilt) ilast=1
#ifdef PLPLOT
      call vplot(ntrcmx,relev(jmin),tshift,cvel(jfilt),tbar, &
      sigmam(jfilt),frq(jfilt),lineid,tyear,tday,ilast,mdim)
#else
      call gvplot(ntrcmx,relev(jmin),tshift,cvel(jfilt),tbar, &
      sigmam(jfilt),frq(jfilt),lineid,tyear,tday,ilast)
#endif
!
!...save results in bvas.his history file
      write(3,'(1x,6(e14.7,1x))') frq(jfilt),cvel(jfilt),sigmam(jfilt),&
      semblx(jfilt),hldtbr(jfilt),hldtvr(jfilt)
!...output to sdtout
      write(*,290) &
      frq(jfilt),cvel(jfilt),sigmam(jfilt),semblx(jfilt),ncnt
  290 format(5x,"Freq=",f6.1,"  Vel.=",f7.1," m/s    +/-", &
      f5.0,"  SEMB=",f8.5,1x,i5," golden searches")

!
      close(iunit1)
!
!...clear output file if iskp=-1 (saves disk space)...
      if(iskp.lt.0) then
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
      endif
!
!...End of filter loop----------------------------------------
  300 continue
      write(*,*) "Saving Semblance: semblance.dat"
!
!...close semblance.dat file
      close(15)
!
#ifndef PLPLOT
      bz=0
      write(cmd,'(''gnuplot bvasqc.gp'',a)') bz
      write(6,*) "Generating bvasqc.ps . . . "
      call runcmd(cmd)
#endif
!
      write(io11,'(50(1h-))')
      write(io11,*)'  BODY WAVE DISPERSION CURVE'
      write(io11,*)'  (Golden Section Search)'
      write(io11,1103)
1103  format(/5x,' Frequency',5x,'Phase Vel.', &
      5x,'+/- m/s', &
      5x,' Semblance',7x,'Tbar',7x,'Tvar'/)
!
      do 400 jfilt=1,nfilt
      write(io11,1104)  frq(jfilt),cvel(jfilt),sigmam(jfilt), &
      semblx(jfilt),hldtbr(jfilt),hldtvr(jfilt)
1104  format(5x,f10.2,5x,f10.2,3x,f10.6,5x,f10.4,1x,f10.4,1x, &
      f10.4)
!
  400 continue
!
!...summary plot
      ilast2=1
!
#ifdef PLPLOT
      call vplot2(nfilt,frq,cvel,sigmam,semblx,emin,emax, &
      lineid,tyear,tday,ilast2,mdim)
#else
      call gvplot2(nfilt,frq,cvel,sigmam,semblx,emin,emax, &
      lineid,tyear,tday)

      bz=0
      write(6,*) "Generating bvas.ps . . . "
      write(cmd,'(''gnuplot bvas.gp'',a)') bz
      call runcmd(cmd)
#endif
!
      call fgnuplot()
!
      write(*,'('' '')')
      write(6,240) ntrace,outlst
240   format(' Traces Processed = ',i5/ &
      ' LISTING File: ',a)
      write(*,*) 'History File: bvas.his'
      write(*,*) 'POSTSCRIPT Files: bvas.ps and bvasqc.ps'
#ifndef PLPLOT
      write(*,*) 'GNUPLOT Files: bvas.gp, bvasqc.gp'
#endif
!
      bz=0
      idelay=1
      call sleep(idelay)
      write(cmd,'('' touch CLRplot.gp'',a)') bz
      call runcmd(cmd)
      call sleep(idelay)
      write(cmd,'('' touch CNTplot.gp'',a)') bz
      call runcmd(cmd)
#ifndef PLPLOT
      call sleep(idelay)
      write(cmd,'('' touch bvas.gp'',a)') bz
      call runcmd(cmd)
      call sleep(idelay)
      write(cmd,'('' touch bvasqc.gp'',a)') bz
      call runcmd(cmd)
#endif
      call sleep(idelay)
      write(cmd,'('' touch clrplot.png'',a)') bz
      call runcmd(cmd)
      call sleep(idelay)
      write(cmd,'('' touch cntplot.png'',a)') bz
      call runcmd(cmd)
      call sleep(idelay)
      write(cmd,'('' touch bvas.ps'',a)') bz
      call runcmd(cmd)
      call sleep(idelay)
      write(cmd,'('' touch bvasqc.ps'',a)') bz
      call runcmd(cmd)

      write(*,*) "OUTPUT FILES: "
      write(*,*) "MSHplot.gp, GRYplot.gp, CLRplot.gp CNTplot.gp "
      write(*,*) "clrplot.png, cntplot.png"
      write(*,*) "bvas.ps bvasqc.ps"

!
      close(io11)
      close(iunit1)
      close(iunit2)
      close(7)
      close(3)
      stop
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: getprm.f                                     |
! |------------------------------------------------------------|
! | get input parameters or help                               |
! --------------------------------------------------------------
      subroutine GETPRM(nargsx,infil, &
      emin,emax,vmin,vmax,nvel,fmin,fmax,delf,bwidth,iskp,ivscn)
!
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9
      character*40      arg10,arg11,arg12
!
!
!...get input/output file names...
!   |--------------------------------------------|
!   | file naming convention                     |
!   | application prefix 4 characters "bxxx"     |
!   | data set suffix 4 characters "dddd"        |
!   | file name = bxxxdddd.seg                   |
!   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
!
!cc---LINUX VERSION
      nargsx=iargc()
      karg=-1
!...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bvas: Measure dispersion (velocity as a  |'
      write(*,*) '| function of frequency in a VSP)          |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*) ' bvas infile emin emax vmin vmax nvel . . .'
      write(*,*) '          fmin fmax delf bwd iskp ivscn'
      write(*,*) ' '
      write(*,*) 'infile  =input file name '
      write(*,*) 'emin    =minimum receiver elevation (float) '
      write(*,*) 'emax    =maximum receiver elevation (float) '
      write(*,*) 'vmin    =minimum velocity '
      write(*,*) 'vmax    =maximum velocity '
      write(*,*) 'nvel    =number of velocity increments'
      write(*,*) 'fmin    =minimum frequency Hz'
      write(*,*) 'fmax    =maximum frequency Hz '
      write(*,*) 'delf    =frequency increment Hz '
      write(*,*) 'bwd     =filter bandwidth Hz '
      write(*,*) 'iskp    =skip filtering (if files already exist)'
      write(*,*) '          1=YES  0=NO (-1=NO and delete when done)'
      write(*,*) 'ivscn   =output velocity scan data sets '
      write(*,*) '          1=YES  0=NO '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a)') infil
      else
      call getarg(1,infil)
      endif

!
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter: emin'
      read(*,*) emin
      else
      call getarg(2,arg2)
      read(arg2,'(f7.0)') emin
      endif
!
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  enter: emax'
      read(*,*) emax
      else
      call getarg(3,arg3)
      read(arg3,'(f7.0)') emax
      endif
!
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  enter: vmin'
      read(*,*) vmin
      else
      call getarg(4,arg4)
      read(arg4,'(f8.0)') vmin
!     vmin=float(ivmin)
      endif
!
      if(nargsx.lt.(6+karg)) then
      write(*,*)'  enter: vmax'
      read(*,*) vmax
      else
      call getarg(5,arg5)
      read(arg5,'(f8.0)') vmax
!     vmax=float(ivmax)
      endif
!
      if(nargsx.lt.(7+karg)) then
      write(*,*)'  enter: nvel'
      read(*,*) nvel
      else
      call getarg(6,arg6)
      read(arg6,'(i5)') nvel
      endif
      if(nvel.lt.3) nvel=3
!
      if(nargsx.lt.(8+karg)) then
      write(*,*)'  enter: fmin'
      read(*,*) fmin
      else
      call getarg(7,arg7)
      read(arg7,'(f7.0)') fmin
      endif
!
      if(nargsx.lt.(9+karg)) then
      write(*,*)'  enter: fmax'
      read(*,*) fmax
      else
      call getarg(8,arg8)
      read(arg8,'(f7.0)') fmax
      endif
!
      if(nargsx.lt.(10+karg)) then
      write(*,*)'  enter: delf'
      read(*,*) delf
      else
      call getarg(9,arg9)
      read(arg9,'(f7.0)') delf
      endif
!
      if(nargsx.lt.(11+karg)) then
      write(*,*)'  enter: filter bandwidth in Hz'
      read(*,*) bwidth
      else
      call getarg(10,arg10)
      read(arg10,'(f6.0)') bwid
      bwidth=bwid
!c      bwidth=float(ibwid)
      endif

!
      if(nargsx.lt.(12+karg)) then
      write(*,*)'  enter: iskp (skip filter) 1=YES 0=NO'
      read(*,*) iskp
      else
      call getarg(11,arg11)
      read(arg11,'(i5)') iskp
      endif
!
      if(nargsx.lt.(13+karg)) then
      write(*,*)'  enter: ivscn (output vscn datasets) 1=YES 0=NO'
      read(*,*) ivscn
      else
      call getarg(12,arg12)
      read(arg12,'(i5)') ivscn
      endif
      return
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: lstprm.f                                     |
! |------------------------------------------------------------|
! | echo check of input parameters, writes to a history file   |
! --------------------------------------------------------------
!
      subroutine LSTPRM(io11,ntrace,ifaz,npoles, &
      emin,emax,vmin,vmax,nvel,fmin,fmax,delf,bwidth,iskp,ivscn, &
      aplc,outlst,outfil,infil)
      character*4       aplc
      character*40      outlst,outfil,infil
!
!...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
!
      write(io11,'('' Program bvas  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil

      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   emin= '',f10.2)') emin
      write(io11,'(''   emax= '',f10.2)') emax
      write(io11,'(''   vmin= '',f10.2)') vmin
      write(io11,'(''   vmax= '',f10.2)') vmax
      write(io11,'(''   nvel= '',i5)')    nvel
      write(io11,'(''   fmin= '',f10.2)') fmin
      write(io11,'(''   fmax= '',f10.2)') fmax
      write(io11,'(''   delf= '',f10.2)') delf
      write(io11,'(''   bwd = '',f10.2)') bwidth
      write(io11,'(''   iskp= '',i5)')    iskp
      write(io11,'(''  ivscn= '',i5)')    ivscn
      write(io11,*) ' HARDWIRED PARAMETERS '
      if(ifaz.eq.0) then
      write(io11,*) 'Filters are Zero Phase'
      else
      write(io11,*) 'Filters are Minimum Phase'
      endif
      write(io11,'(''  npoles= '',i5)')    npoles
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
!
      return
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: stage1.f                                     |
! |------------------------------------------------------------|
! | Load Geometry, compute angles                              |
! --------------------------------------------------------------
!
      subroutine STAGE1(npts,ntrace,s1,depth,angle,cosa,io11,ndim, &
      relev,infil,icall1,iunit1,fsamin,mdim)
      include '../include/bsegy.f90'
!
      real*4            depth(*),angle(*),cosa(*),relev(*)
!
      real*4            s1(*)
      character*40      infil
      common /header/ hd
!
      jrec = 0
!
!...BEGIN LOOP====================================....
!
      do 600 jrec=1,ntrace
!
!...input a trace
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
      if(iexit.eq.1) go to 600
!
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
!
!
      irz=int(rz)
!
!...compute geometry for each trace, jrec: depth,angle,cos(angle)
      if(jrec.gt.mdim) then
      write(*,*)' ABORT--number of traces>mdim dimension'
      stop
      endif
!
      relev(jrec)=rz
      depth(jrec)=abs(sz-sd-rz)
      angle(jrec)=acos(depth(jrec)/range)
      angle(jrec)=angle(jrec)*(180./3.1415926)
      cosa(jrec)=depth(jrec)/range
!
!...END LOOP========================================....
600   continue
      write(io11,'(/''   GEOMETRY:''/)')
      write(io11,123)
  123 format(5x,'Trace',5x,'Depth',5x,'Angle',5x,'Cos(0)'/)
      do 32 jrec=1,ntrace
      write(io11,'(5x,i5,5x,f5.2,5x,f5.1,5x,f6.4)') &
      jrec,depth(jrec),angle(jrec),cosa(jrec)
!
   32 continue
      write(*,*) ' '
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: filter.f                                     |
! |------------------------------------------------------------|
! | Applies band-pass filter and scaling                       |
! --------------------------------------------------------------
!
      subroutine FILTER(npts,ntrace,s1,infil,iunit1, &
      icall1,fsamin,s2,outfil,iunit2,icall2, &
      itype,bwidth,npoles,fcenter,tmin1,tmax1,ndim,ifaz)
!
      include '../include/bsegy.f90'
!
      real*4            s1(*),s2(*)
      character*40      infil , outfil
      common /header/ hd
!
      do 200 jrec=1,ntrace
!
!...input a trace and check dimensions...
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
!...filter
      call ARMA(npts,s1,s2,itype,fcenter,bwidth,npoles,fsamin,ifaz)
!
!...trace equalize
      call L2NORM(npts,s2,s1,fsamin,tmin1,tmax1)
!
!...output trace
      CALL BSEGOUT(iunit2, &
      icall2,npts,s1,fsamin,outfil,jrec)
!
  200 continue
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: velscn.f                                     |
! |------------------------------------------------------------|
! | Semblance scan of velocities                               |
! --------------------------------------------------------------
!
      subroutine VELSCN(npts,s1,infil,iunit1,fcenter, &
      icall1,fsamin,s2,depth,cosa,angle,ivscn,ndim, &
      vel,sembl,jmin,jmax,s4,s5,ntrhld,icall7,SF)
!
      include '../include/bsegy.f90'
!
      real*4            depth(*),angle(*),cosa(*)
      real*4            s1(*),s2(*),s4(*),s5(*)
      character*40      infil,outfil
      common /header/ hd
!
!...clear old file and name new
      if(icall7.eq.0) then
      ifcnter=int(fcenter*SF)
      write(outfil,'(''v'',i5.5,''.seg'')') ifcnter
      open(7,file=outfil,access='direct',recl=240)
      close(7,status='delete')
      endif
!
      ntrc=jmax-jmin+1
      bar2=float(ntrc)/50.
      ibar2=int(bar2+0.5)
      invbr2=int(1./bar2+0.5)
!
!...clear accumulators
      do 50 jt=1,npts
      s4(jt)=0.
   50 s5(jt)=0.
!
      ntr=0
!
      do 200 jrec=jmin,jmax
      ntr=ntr+1
      ntrhld=ntrhld+1
!
!...input a trace and check dimensions...
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
!...shift trace into buffer s2
      call APLYV(npts,s1,s2,jrec,vel,depth,cosa,angle,fsamin, &
      jmin,jmax)
!
!...velscn data set/////////////////////
!
      if(ivscn.eq.1) then
      call BSEGOUT(7, &
      icall7,npts,s2,fsamin,outfil,ntrhld)
      endif
!
!...compute semblance accumulators
      do 100 jt=1,npts
! stack trace
      s4(jt)=s4(jt)+s2(jt)
! squared trace
      s5(jt)=s5(jt)+s2(jt)*s2(jt)
  100 continue
!
  200 continue
! Next trace
!
!...velscn data set separator traces/////////////////////
!
      if(ivscn.eq.1) then
      do 127 jt=1,npts
  127 s2(jt)=0.
!
      do 128 jrec=1,2
      ntrhld=ntrhld+1
      call BSEGOUT(7, &
      icall7,npts,s2,fsamin,'velscn.seg',ntrhld)
  128 continue
      endif
!
!
!...compute semblance
      sstack=0.0
      estack=0.0
      do 130 jt=1,npts
      sstack=sstack + s4(jt)*s4(jt)
      estack=estack + s5(jt)
  130 continue
      sembl=(sstack/estack)/float(ntrc)
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: zrange.f                                     |
! |------------------------------------------------------------|
! | Finds depth range based first and last traces              |
! --------------------------------------------------------------
!
      subroutine ZRANGE(ntrace,emin,emax,jmin,jmax,depth,relev, &
      zmin,zmax,io11)
      real*4            depth(*),relev(*)
!
      eps1=1.0E+20
      eps2=1.0E+20
      zmin=depth(1)
      do 10 j=1,ntrace
      test=abs(abs(relev(j))-abs(emin))
      if(test.lt.eps1) then
      jmin=j
      zmin=depth(j)
      eps1=test
      endif
      test=abs(abs(relev(j))-abs(emax))
      if(test.lt.eps2) then
      jmax=j
      zmax=depth(j)
      eps2=test
      endif
   10 continue
      if(zmin.gt.zmax) then
      hold=zmax
      zmax=zmin
      zmin=hold
      endif
      write(io11,'(''   emin= '',f8.1,'' Trace#='',i5)') emin,jmin
      write(io11,'(''   emax= '',f8.1,'' Trace#='',i5)') emax,jmax
      write(io11,'(''   zmin= '',f8.1)') zmin
      write(io11,'(''   zmax= '',f8.1)') zmax
!
!...strategy for possible reverse trace order
      if(jmin.gt.jmax) then
      jhold=jmin
      jmin=jmax
      jmax=jhold
      endif
!
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: vshft.f                                      |
! |------------------------------------------------------------|
! | Shifts a trace by interpolation method                     |
! --------------------------------------------------------------
!
      subroutine VSHFT(npts,s1,s2,dt,fsamin)
      real*4            s1(*),s2(*)
!
!...clear output trace
      do 70 jt=1,npts
   70 s2(jt)=0.0
!
      tmax=(npts-1)*fsamin
      do 80 jt=1,npts
      tpic=(jt-1)*fsamin+dt
      if(tpic.lt.tmax .and. tpic.ge.0.) then
      s2(jt)=VALXX(tpic,s1,fsamin)
      else
      s2(jt)=0.0
      endif
   80 continue
!
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: valxx.f                                      |
! |------------------------------------------------------------|
! | Function interpolates a value at time tpic                 |
! --------------------------------------------------------------
!
      real*4 function VALXX(tpic,s1,fsamin)
      real*4            s1(*)
!
      fi=tpic/fsamin+1.
      i=int(fi)
      j=i+1
      VALXX=((s1(j)-s1(i))/fsamin)*(tpic-float(i-1)*fsamin)+s1(i)
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: aplyv.f                                      |
! |------------------------------------------------------------|
! | Apply velocity shift to trace                              |
! | Note: argument angle is taken between the vertical and a   |
! | line connecting the source and receiver. cosa is the cosine|
! | of angle (see stage1).                                     |
! | If angle > about 88 degrees, cosa -->zero, and so, to      |
! | avoid a divide by zero, skip those cases.  In essence, the |
! | geophone would be at the same elevation as the source.     |
! | The cosa correction is to make this a vertical correction. |
! --------------------------------------------------------------
!
      subroutine APLYV(npts,s1,s2,jrec,vel,depth,cosa,angle,fsamin, &
      jmin,jmax)
      real*4            s1(*),s2(*),depth(*),cosa(*),angle(*)
!
!...compute bias shift, dtbias
      dtbias=(depth(jmin)/vel)/cosa(jmin)
      dtbs2=(depth(jmax)/vel)/cosa(jmax)
      if(dtbs2.gt.dtbias) dtbias=dtbs2
!
!...compute time shift/////////////
      if(angle(jrec).lt.88.) then
      dt=((depth(jrec)/vel)/cosa(jrec))-dtbias
!
      call VSHFT(npts,s1,s2,dt,fsamin)
!
!/////over 88 degrees
      else
      do 222 jt=1,npts
  222 s2(jt)=0.
      endif
      return
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: GS.f                                         |
! |------------------------------------------------------------|
! | Golden Section Search for MAXIMUM SEMBLANCE                |
! | (note, sense is opposite of usual minimization, here a     |
! |  maximization instead)                                     |
! |                                                            |
! | Tolerance (tol) should be set at about sqrt(real*4 prec.)  |
! | Calls Functions:  fnewx.f and F.f                          |
! | Assumes Interval Bracketed in main program:                |
! | [a   b   c] where fb>fa and fb>fc (maximum between a and c)|
! | fnewx.f finds next search coordinate, x                    |
! | [a   b x c]  iblock=1 case                                 |
! |            or                  iblock is a switch          |
! | [a x b   c]  iblock=0 case                                 |
! |                                                            |
! | Depending on fb and fx, new interval is chosen.            |
! | fnewx.f splits interval in golden ratio.                   |
! | F.f is the function to be maximized (semblance)            |
! |                                                            |
! --------------------------------------------------------------
!
      real*4 function GS(a,b,c,x, &
      npts,s1,infil,iunit1,icall1,fsamin,s2,depth,cosa, &
      angle,jmin,jmax,s4,s5,ncnt)
      parameter (tol=1.0E-04)
!
      real*4            depth(*),angle(*),cosa(*)
      real*4            s1(*),s2(*),s4(*),s5(*)
      character*40      infil
!
!...find initial x
      ncnt=0
      x=fnewx(a,b,c,tst,iblock)
!
!...initial function calls
      fb=F(b,npts,s1,infil,iunit1,icall1,fsamin,s2,depth,cosa, &
      angle,ndim,jmin,jmax,s4,s5)
      fx=F(x,npts,s1,infil,iunit1,icall1,fsamin,s2,depth,cosa, &
      angle,ndim,jmin,jmax,s4,s5)
!
!...begin loop, stop when converged
100   if(abs(tst).gt.tol*abs(b)) then
      ncnt=ncnt+1
        if(iblock.eq.1) then
          if(fb.lt.fx) then
          a=b
          b=x
!         c=c
          fb=fx
          else
!         a=a
!         b=b
          c=x
          endif
        else
          if(fb.lt.fx) then
          c=b
          b=x
!         a=a
          fb=fx
          else
!         c=c
!         b=b
          a=x
          endif
        endif
      x=fnewx(a,b,c,tst,iblock)
      fx=F(x,npts,s1,infil,iunit1,icall1,fsamin,s2,depth,cosa, &
      angle,ndim,jmin,jmax,s4,s5)
      go to 100
      endif
!...end of loop
!
!...take best result
      if(fb.gt.fx) then
      GS=fb
      x=b
      else
      GS=fx
      endif
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: newx.f                                       |
! |------------------------------------------------------------|
! | Function to determine next new x       [a   b x c]iblock=1 |
! |                                   or   [a x b   c]iblock=0 |
! --------------------------------------------------------------
      real*4 function fnewx(a,b,c,tst,iblock)
      parameter (g1=.61803399, g2=.3819661)
      bc=abs(c-b)
      ab=abs(b-a)
      if(bc.gt.ab) then
      tst=bc
      iblock=1
      fnewx=g2*bc+b
!     fnewx=g1*b+g2*c
      else
      tst=ab
      iblock=0
      fnewx=b-g2*ab
!     fnewx=g1*b+g2*a
      endif
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: F.f                                          |
! |------------------------------------------------------------|
! | The external function, F(x), to MAXIMIZE                   |
! --------------------------------------------------------------
!
      real*4 function F(vel,npts,s1,infil,iunit1, &
      icall1,fsamin,s2,depth,cosa,angle,ndim, &
      jmin,jmax,s4,s5)
!
      include '../include/bsegy.f90'
!
      real*4            depth(*),angle(*),cosa(*)
      real*4            s1(*),s2(*),s4(*),s5(*)
      character*40      infil
      common /header/ hd
!
      ntrc=jmax-jmin+1
!
!...clear accumulators
      do 50 jt=1,npts
      s4(jt)=0.
   50 s5(jt)=0.
!
      ntr=0
!
      do 200 jrec=jmin,jmax
      ntr=ntr+1
!
!...input a trace ...
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
!...shift trace into buffer s2
      call APLYV(npts,s1,s2,jrec,vel,depth,cosa,angle,fsamin, &
      jmin,jmax)
!
!...compute semblance accumulators
      do 100 jt=1,npts
      s4(jt)=s4(jt)+s2(jt)
      s5(jt)=s5(jt)+s2(jt)*s2(jt)
  100 continue
!
  200 continue
!
!...compute semblance
      sembl=0.
      do 300 jt=1,npts
      if(s5(jt).gt.0.0) then
      sembl=sembl+(s4(jt)*s4(jt))/(s5(jt))
      endif
  300 continue
      sembl=sembl/(float(npts)*float(ntrc))
      F=sembl
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: tvar.f                                       |
! |------------------------------------------------------------|
! |          Function to measure time shifts wrt 1st trace     |
! | tbar=average relative shift                                |
! | tvar=variance about tbar                                   |
! | tshift=vector with shifts                                  |
! --------------------------------------------------------------
!
      real*4 function tvar(vel,npts,s1,infil,iunit1, &
      icall1,fsamin,s2,depth,cosa,angle,ndim,io11, &
      jmin,jmax,s4,tshift,fcenter,tbar)
!
      include '../include/bsegy.f90'
!
      real*4            depth(*),angle(*),cosa(*)
      real*4            s1(*),s2(*),s4(*),tshift(*)
      character*40      infil
      common /header/ hd
!
      ntrc=jmax-jmin+1
!
!
!...compute limits on signal of post vel corrected stack trace
      call fsignal(vel,npts,s1,infil,iunit1,icall1,fsamin, &
      s2,depth,cosa,angle,jmin,jmax,s4,fcenter, &
      fmax,kmax,jtmin,jtmax)

!...output signal envelope maximum energy, time, innerproduct window
      tkmax=float(kmax-1)*fsamin
      tjtmin=float(jtmin-1)*fsamin
      tjtmax=float(jtmax-1)*fsamin
      write(io11,291) fmax,tkmax,tjtmin,tjtmax
  291 format(" Maximum Signal=",e14.7," at ",f8.4, &
      "sec"/" Innerproduct Gate: ",f8.4," to ", &
      f8.4," sec.")
!
!...input first trace as a reference
      jrec=jmin
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
!...shift trace into buffer s4
      call APLYV(npts,s1,s4,jrec,vel,depth,cosa,angle,fsamin, &
      jmin,jmax)
!
!...compute norm of reference trace
      sum4=0.
      do 98 jt=jtmin,jtmax
   98 sum4=sum4+s4(jt)*s4(jt)
      if(sum4.ne.0.) then
      fnorm4=sqrt(sum4)
      else
      fnorm4=1.
      endif
!
      ntr=0
!
      do 200 jrec=jmin,jmax
      ntr=ntr+1
!
!...input a trace ...
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
!...shift trace into buffer s2
      call APLYV(npts,s1,s2,jrec,vel,depth,cosa,angle,fsamin, &
      jmin,jmax)
!
!...compute norms and inner products
      sum2=0.
      sum24=0.
      do 100 jt=jtmin,jtmax
      sum2=sum2+s2(jt)*s2(jt)
      sum24=sum24+s2(jt)*s4(jt)
  100 continue
      if(sum2.ne.0.) then
      fnorm2=sqrt(sum2)
      else
      fnorm2=1.
      endif
      fcosan=sum24/(fnorm4*fnorm2)
      if(fcosan.gt.1.0) fcosan=1.0
      engle=acos(fcosan)
      tshift(ntr)=engle/(2.*3.1415926*fcenter)
!
  200 continue
!
!...compute variance
      tbar=0.
      do 150 j=1,ntrc
  150 tbar=tbar+tshift(j)
      tbar=tbar/float(ntrc)
      sum=0.
      do 160 j=1,ntrc
  160 sum=sum+(tshift(j)-tbar)**2
      tvar=sqrt(sum/float(ntrc))
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: fsignal.f                                    |
! |------------------------------------------------------------|
! | finds limits on signal of stacked trace, returns limits    |
! |  jtmin, jtmax for innerproduct calculation in function tvar|
! |                                                            |
! |This subroutine solves the problem of taking in too much    |
! |noise in the innerproduct calculation, which degrades the   |
! |determination of the relative time shifts                   |
! --------------------------------------------------------------
      subroutine fsignal(vel,npts,s1,infil,iunit1,icall1, &
      fsamin,s2,depth,cosa,angle,jmin,jmax,s4,fcenter, &
      fmax,kmax,jtmin,jtmax)
!
      include '../include/bsegy.f90'
!
      real*4            depth(*)
      real*4            s1(*),s2(*),s4(*)
      real*4            cosa(*),angle(*)
      character*40      infil
      common /header/ hd
!
!...zero the stacked trace
      do 50, jt=1,npts
   50 s4(jt)=0.0
!
!...compute the stacked trace after applying the velocity solution
      ntr=0
      do 100, jrec=jmin,jmax
      ntr=ntr+1
!
!...input a trace ...
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!
!...shift trace into buffer s2
      call APLYV(npts,s1,s2,jrec,vel,depth,cosa,angle,fsamin, &
      jmin,jmax)
!
      do 200, jt=1,npts
  200 s4(jt)=s4(jt)+s2(jt)
  100 continue
!
!...compute energy in stacked trace
      do 300, jt=1,npts
  300 s4(jt)=s4(jt)*s4(jt)
!
!...compute smoothing length as period of filter
      iwide=int((1.0/fcenter)/fsamin+.5)
!
!...apply box car smoother to energy trace --> envelope
!   energy envelope in s2
      isign=-1
      ntimes=1
      call boxit(npts,s4,s2,s1,iwide,isign,ntimes)
!
!...find maximum of energy envelope
      fmax=-999999.
      do 400, jt=1,npts
      if (s2(jt).gt.fmax) then
      fmax=s2(jt)
      kmax=jt
      endif
  400 continue
!
!...set threshold to establish signal region
      thresh=0.5*fmax
!
!...determine signal region
      jtmin=1
      jtmax=npts
      do 500, jt=1,kmax
      if(s2(jt).gt.thresh) then
      jtmin=jt
      goto 501
      endif
  500 continue
  501 continue
      do 502, jt=kmax,npts
      if(s2(jt).lt.thresh) then
      jtmax=jt
      goto 503
      endif
  502 continue
  503 continue
      return
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http:ggcgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: fgnuplot.f                                   |
! |------------------------------------------------------------|
! | generate an GNUPLOT programs to display dispersion data    |
! |                                                            |
! | iout=unit to write to.                                     |
! --------------------------------------------------------------
      subroutine fgnuplot()
      integer*1       bz
      character*1024  cmd
      iout=9

! note that $ supresses line CR/LF in Fortran writes
      open(unit=iout, file='GRYplot.gp',access='sequential')
      write(iout,*) "# Generated by bvas.F90   "
      write(iout,*) "set pm3d   "
      write(iout,*) "#set view map   "
      write(iout,*) "set dgrid3 200,200 gauss 1, 1"
      write(iout,'(a,$)') "set palette model RGB defined "
      write(iout,'(a,$)') "(0 '#aaaaaa', 1.0 'black')"
      write(iout,'(a)') " "
      write(iout,*) "set grid   "
      write(iout,*) "set mxtics   "
      write(iout,*) "set mytics   "
      write(iout,*) "set nokey   "
      write(iout,*) "set xlabel('Frequency (Hz)')  "
      write(iout,*)"set ylabel('Phase Velocity mgs')"
      write(iout,*) "set title('Semblance') "
      write(iout,*) "splot 'semblance.dat' w pm3d  "
      write(iout,*) "set view 38,44   "
      write(iout,*) "#set terminal png size 1024,768 "
      write(iout,*) "#set output 'gryplot.png'  "
      write(iout,*) "#replot  "
      close(iout)

      open(unit=iout, file='MSHplot.gp',access='sequential')
      write(iout,*) "# Generated by bvas.F90     "
      write(iout,*) "set dgrid3 100,100 gauss 1, 1"
      write(iout,*) "set grid   "
      write(iout,*) "set mxtics   "
      write(iout,*) "set mytics   "
      write(iout,*) "set nokey   "
      write(iout,*) "set xlabel('Frequency (Hz)')  "
      write(iout,*)"set ylabel('Phase Velocity mgs')"
      write(iout,*) "set title('Semblance') "
      write(iout,*) "set view 38,44   "
      write(iout,'(a,$)') "set hidden3d offset 0 "
      write(iout,'(a,$)') "trianglepattern 2 #for mesh plot)"
      write(iout,'(a)') " "
      write(iout,*) "splot 'semblance.dat' w lines "
      write(iout,*) "#set terminal png size 1024,768 "
      write(iout,*) "#set output 'mshplot.png'  "
      write(iout,*) "#replot  "
      close(iout)

      open(unit=iout, file='CLRplot.gp',access='sequential')
      write(iout,*) "# Generated by bvas.F90   "
      write(iout,*) "set pm3d   "
      write(iout,*) "set view map   "
      write(iout,*) "unset surface   "
      write(iout,*) "set dgrid3 300,300 gauss 1, 1"
      write(iout,'(a,$)') "set palette model RGB defined "
      write(iout,'(a,$)') "(0 'violet', .20 'blue',  .40 'green',"
      write(iout,'(a,$)')".60 'yellow', .80 'orange', 1.00 'red')"
      write(iout,'(a)') " "
      write(iout,*) "set grid   "
      write(iout,*) "set mxtics   "
      write(iout,*) "set mytics   "
      write(iout,*) "set nokey   "
      write(iout,*) "set xlabel('Frequency (Hz)')  "
      write(iout,*)"set ylabel('Phase Velocity mgs')"
      write(iout,*) "set title('Semblance') "
      write(iout,*) "splot 'semblance.dat'  "
      write(iout,*) "set terminal png size 1024,768 "
      write(iout,*) "set output 'clrplot.png'  "
      write(iout,*) "replot   "
      close(iout)
      bz=0
      write(cmd,'(''gnuplot CLRplot.gp '',a)') bz
      write(*,*) "please wait, will display and then save. . ."
      write(6,*) "Generating Color Plot clrplt.png . . . "
      call runcmd(cmd)

      open(unit=iout, file='CNTplot.gp',access='sequential')
      write(iout,*) "# Generated by bvas.F90   "
      write(iout,*) "set view map   "
      write(iout,*) "unset surface   "
      write(iout,*) "set contour base "
      write(iout,*) "set cntrparam levels 10 "
      write(iout,*) "set cntrparam bspline "
      write(iout,*) "set clabel '%0.2f' "
      write(iout,*) "set dgrid3 100,100 gauss 1, 1 "
      write(iout,*) "set grid   "
      write(iout,*) "set mxtics   "
      write(iout,*) "set mytics   "
      write(iout,*) "set key outside right   "
      write(iout,*) "set xlabel('Frequency (Hz)')  "
      write(iout,*)"set ylabel('Phase Velocity mgs')"
      write(iout,*) "set title('Semblance') "
      write(iout,*) "splot 'semblance.dat' w lines "
      write(iout,*) "set terminal png size 1024,768 "
      write(iout,*) "set output 'cntplot.png'  "
      write(iout,*) "replot   "
      close(iout)
      write(cmd,'(''gnuplot CNTplot.gp '',a)') bz
      write(*,*) "please wait, will display and then save. . ."
      write(6,*) "Generating Contour Plot cntplt.png . . . "
      call runcmd(cmd)
      return
      end
#ifdef PLPLOT
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: vplot.f                                      |
! |------------------------------------------------------------|
! |REQUIRES PLPLOT:                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |------------------------------------------------------------|
! | velocity  plot  For QC (good is horizontal line with points|
! |                       cluster close to line)               |
! |                                                            |
! |   x = geophone elevation                                   |
! |   y = relative time shift in seconds after velocity applied|
! |  tbvar= average relative time shift                        |
! --------------------------------------------------------------
!
      subroutine vplot(npts,x,y,cvel,tbar,vvar,freq, &
      lineid,tyear,tday,ilast,mdim)
      use plplot
      real*4 x(npts),y(npts)
      real*8 ybar(2),xbar(2)
      real*8 xd(mdim),yd(mdim)
      real*8 xmn,xmx,ymn,ymx
      real*8 xmin,xmax,ymin,ymax
      real*8 fdef,fscale,fzero
      real*8 fmargin,aspect,yjust,xjust
      real*8 fwidth
!
      character*14 pltfil
      character*80 titl
      character*40 xlbl
      character*40 ylbl
      character*4 lineid
      integer*2 tyear,tday
      data ifirst /1/
!...copy to double
      do 889, j=1,npts
      xd(j)=x(j)
889   yd(j)=y(j)

!
!...open device
      if (ifirst.eq.1) then
!  Portrait
      izero=0
      ione=1
      call plscolor(izero)
      call plsori(ione)
!...aspect ratio needed for portrait, unless you don't mind
!   plots which are squished in the vertical dimension.
!   aspect=8.5/11.0
          fmargin=0.
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('ps')
          write(pltfil,'(''bvasqc.ps'')')
!
      call plsdidev(fmargin,aspect,yjust,xjust)
!
      call plsfnam(pltfil)
!
!...initialize plot
      nx=1
      ny=1
      izero=0
      call plinit()
      call plssub(nx,ny)
      call plfontld(izero)
      ifirst=0
!
      endif
!-------end ifirst test-----------------
!
!...convert y to msec from seconds
      fscl=1000.
      do 33, j=1,npts
33    yd(j)=yd(j)*fscl
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 10, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
!      ymax=max(ymax,.03*fscl)
      ymin=min(ymin,0.0)
  10  continue
!
!...simplify vertical scaling to make comparison easier
!   two possible scales
      if(tbar*fscl.gt.(.015*fscl)) then
      ymax=.06*fscl
      else
      ymax=.03*fscl
      endif
!
!...set environment, window and viewport
!      call plenv(xmin,xmax,ymin,ymax,0,0)
      izero=0
      ione=1
      iwidth=1
      fwidth=1.0
      fdef=0.
      fscale=0.6
      call pladv(izero)
      call plschr(fdef,fscale)
      call plprec(ione,izero)
      call plssym(fdef,fscale)
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      xmn=0.1
      xmx=.95
      ymn=0.6
      ymx=.95
      call plvpor(xmn,xmx,ymn,ymx)
      call plwind(xmin,xmax,ymin,ymax)
      fzero=0.
      izero=0
      call plbox('bcnst',fzero,izero,'bcnstv',fzero,izero)
!
      write(titl,'('' FREQ='',f6.1,'' Hz'', &
      '' Velocity='',f8.4,'' +/-'',f6.1, &
      '' m/s ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       freq,cvel,vvar,lineid,tyear,tday
      write(xlbl,'(''Geophone Elev. (m) '')')
      if (fscl.gt.2) then
      write(ylbl,'(''Post Velocity Relative Time (msec)'')')
      else
      write(ylbl,'(''Post Velocity Relative Time (s)'')')
      endif
      call pllab(xlbl,ylbl,titl)
      icode=9
      call plpoin(xd,yd,icode)
!
!...plot calculated
      xbar(1)=xmin
      xbar(2)=xmax
      ybar(1)=tbar*fscl
      ybar(2)=tbar*fscl
      call plline(xbar,ybar)
      if(ilast.eq.1) then
      call plend()
      endif
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: vplot2.f                                     |
! |------------------------------------------------------------|
! |REQUIRES PLPLOT:                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |------------------------------------------------------------|
! | velocity  plot  SUMMARY                                    |
! |                                                            |
! |   x = frequency                                            |
! |   y = phase velocity                                       |
! |   sigmam = 1 standard deviation, phase velocity            |
! |   semblx = semblance values for velocity determination     |
! --------------------------------------------------------------
!
      subroutine vplot2(npts,x,y,sigmam,semblx,emin,emax, &
      lineid,tyear,tday,ilast,mdim)
      use plplot
      real*4 x(npts),y(npts),sigmam(npts),semblx(npts)
      real*8 xd(npts),yd(npts)
      real*8 fdef,fscale
      real*8 fmargin,aspect,yjust,xjust
      real*8 xmin,xmax,ymin,ymax,ymin2,ymax2
      real*8 xx,y1,y2
      real*8 fwidth
      character*14 pltfil,pltdev
      character*80 titl
      character*40 xlbl
      character*40 ylbl
      character*4 lineid
      integer*2 tyear,tday
      integer*1 bz
      data ifirst /1/
!
      bz=0
      izero=0
      ione=1

!...copy to double precision for plot
      do 1105, j=1,npts
      xd(j)=x(j)
1105  yd(j)=y(j)
!
!...open device
      if (ifirst.eq.1) then
!  Portrait
      call plscolor(izero)
      call plsori(ione)
!...aspect ratio needed for portrait, unless you don't mind
!   plots which are squished in the vertical dimension.
!   aspect=8.5/11.0
          fmargin=0.
          aspect=0.77
          yjust=0.5
          xjust=0.0
          write(pltdev,'(''ps'',a)') bz
          call plsdev(pltdev)
          write(pltfil,'(''bvas.ps'',a)') bz
!
      call plsdidev(fmargin,aspect,yjust,xjust)
!
      call plsfnam(pltfil)
!
!...initialize plot
      nx=1
      ny=2
      call plinit()
      call plssub(nx,ny)
      call plfontld(izero)
      ifirst=0
!
      endif
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      ymin2=semblx(1)
      ymax2=semblx(1)
      do 10, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
      ymin2=min(semblx(j),ymin2)
      ymax2=max(semblx(j),ymax2)
! set standard scales
      xmin=min(xmin,0.0)
!     ymax=max(ymax,1000)
      ymax=1000.0
      ymin=min(ymin,0.0)
      ymin2=min(ymin2,0.0)
!     ymax2=max(ymax2,1.0)
      ymax2=1.0
  10  continue
!
!...set environment, window and viewport
      izero=0
      ione=1
      iwidth=2
      fwidth=2.0
      fdef=0.
      fscale=0.9
      call plschr(fdef,fscale)
      call plprec(izero,izero)
      call plssym(fdef,fscale)
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      call plenv(xmin,xmax,ymin,ymax,izero,izero)
      write(titl,'(''Velocity (95%conf)'', &
      '' Elevations: '',f6.2,'' to '', &
      f6.2,'' m ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       emin,emax,lineid,tyear,tday
      write(xlbl,'(''Frequency (Hz) '')')
      write(ylbl,'(''Phase Velocity (m/s)'')')
      call pllab(xlbl,ylbl,titl)
      icode=9
      call plpoin(xd,yd,icode)
!
!...plot error bars for velocity, 95% confidence
!      do 50, j=1,npts
!      xx=xd(j)
!      y1=yd(j)+sigmam(j)*1.96
!      y2=yd(j)-sigmam(j)*1.96
!      call plerry(1,xx,y1,y2)
!   50 continue
      call plerry(xd,yd+sigmam*1.96,yd-sigmam*1.96)
!
!...plot semblance values
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      call plenv(xmin,xmax,ymin2,ymax2,izero,izero)
      write(titl,'(''Semblance Values'', &
      '' Elevations: '',f6.2,'' to '', &
      f6.2,'' m ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       emin,emax,lineid,tyear,tday
      write(xlbl,'(''Frequency (Hz) '')')
      write(ylbl,'(''Semblance (unitless)'')')
      call pllab(xlbl,ylbl,titl)
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
!...double precision for plot
      do 703, j=1,npts
703   yd(j)=semblx(j)
      call plline(xd,yd)
!
      if(ilast.eq.1) then
      call plend()
      endif
      return
      end
#else
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: gvplot2.f                                    |
! |------------------------------------------------------------|
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |------------------------------------------------------------|
! | velocity  plot  SUMMARY                                    |
! |                                                            |
! |   x = frequency                                            |
! |   y = phase velocity                                       |
! |   sigmam = 1 standard deviation, phase velocity            |
! |   semblx = semblance values for velocity determination     |
! --------------------------------------------------------------
!
      subroutine gvplot2(npts,x,y,sigmam,semblx,emin,emax, &
      lineid,tyear,tday)
      real*4 x(npts),y(npts),sigmam(npts),semblx(npts)
      character*4 lineid
      integer*2 tyear,tday
      character*80 sxrange,syrange,syrange2
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      ymin2=semblx(1)
      ymax2=semblx(1)
      do 5, j=2,npts
!     xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
!     ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
!     ymin2=min(semblx(j),ymin2)
!     ymax2=max(semblx(j),ymax2)
    5 continue
! overides
      testy=ymax
      xmin=0.0
      ymin=0.0
      if (testy.LT.2000.0) ymax=2000.0
      if (testy.LT.1000.0) ymax=1000.0
      if (testy.LT.500.0) ymax=500.0
      ymin2=0.0
      ymax2=1.0
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax
      write(syrange2,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax2
!
      open(unit=12,file='bvas.gp',access='sequential')
      write(12,*) "# Generated by bvas.F90  Subroutine gvplot2()     "
      write(12,*) "#set terminal x11 persist                         "
      write(12,*) "set terminal postscript size 8.0,10.5 in \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output 'bvas.ps'                              "
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Frequency (Hz)'                       "
      write(12,*) "set ylabel 'Phase Velocity (m/s)                  "
      write(12,*) "set title 'Velocity (95% Conf) Elevations: \"
      write(12,'(F6.2," to ",F6.2,"\")') emin,emax 
      write(12,*) "meters \"
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " '                                                "
      write(12,*) "set multiplot layout 2,1 columnsfirst downwards \"
      write(12,*) "scale .8,.8  offset -.03,-.02                     "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2):($3*1.96) w yerrorbars \"
      write(12,*) "pt 6 ps 1                                         "
      do 10, j=1,npts
   10 write(12,'(3(E14.7,1x))') x(j),y(j),sigmam(j)
      write(12,*) "e "
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Frequency (Hz)'                       "
      write(12,*) "set ylabel 'Semblance (unitless)                  "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange2 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set title 'Semblance Values Elevations: \"
      write(12,'(F6.2," to ",F6.2,"\")') emin,emax 
      write(12,*) "meters \"
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " ' "  
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) ls 1 w l                     "
      do 20, j=1,npts
   20 write(12,'(3(E14.7,1x))') x(j),semblx(j)
      write(12,*) "e "
      write(12,*) "unset multiplot                                   "
      close(12)
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: gvplot.f                                     |
! |------------------------------------------------------------|
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |------------------------------------------------------------|
! | velocity  plot  For QC (good is horizontal line with points|
! |                       cluster close to line)               |
! |                                                            |
! |   x = geophone elevation                                   |
! |   y = relative time shift in seconds after velocity applied|
! |  tbvar= average relative time shift                        |
! --------------------------------------------------------------
!
      subroutine gvplot(npts,x,y,cvel,tbar,vvar,freq, &
      lineid,tyear,tday,ilast)
      real*4 x(npts),y(npts)
      real*4 ybar(2),xbar(2)
      real*4 xmin,xmax,ymin,ymax
      integer*2 tyear,tday
      character*4 lineid
      character*80 sxrange,syrange
      data ifirst /1/
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 5, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
      ymin=min(ymin,0.0)
    5 continue
!scale factor to go from seconds to ms
      fscl=1000.
! overides
!   two possible scales
      if(tbar*fscl.gt.(.015*fscl)) then
      ymax=.06*fscl
      else
      ymax=.03*fscl
      endif
      ymin=ymin*fscl
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax
! plot fit line
      xbar(1)=xmin
      xbar(2)=xmax
      ybar(1)=tbar*fscl
      ybar(2)=tbar*fscl
!--------test if first call-------
      if (ifirst.eq.1) then     
      open(unit=12,file='bvasqc.gp',access='sequential')
      write(12,*) "# Generated by bvas.F90  Subroutine gvplot()      "
      write(12,*) "set terminal postscript size 7.0,5.0 in \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output 'bvasqc.ps'                            "
      ifirst=0
      endif
!------end ifirst test------------
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Geophone Elevation (m)'               "
      write(12,*) "set ylabel 'Post Velocity Relative Time (s)'      "
      write(12,*) "set title 'Frequency=\"
      write(12,'(f6.1," Hz Velocity=",f8.4,"+/- ",f6.1," m/s\")') &
       freq,cvel,vvar
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " '  "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) w p pt 6 ps 1, \"
      write(12,*) " '' u ($1):($2) w l ls 1     "
      do 10, j=1,npts
   10 write(12,'(2(E14.7,1x))') x(j),y(j)*fscl
      write(12,*) "e "
      do 20, j=1,2
   20 write(12,'(2(E14.7,1x))') xbar(j),ybar(j)
      write(12,*) "e "
      if(ilast.EQ.1) then
      close(12)
      endif
      return
      end
#endif
