c |$Id: bvsp.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $              |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bvsp.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | bvsp.f  Travel time inversion of VSP first arrivals        |
c |         (ray bending, but only horizontal layers)          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c |     REFR    Ray trace refracted wave                       |
c |     REFH    Ray trace head (interface) wave                |
c |     DIRECT  Ray trace direct wave                          |
c |     FSTBRK  Determine first arrival time and wave type     |
c |     DERIV   Simple forward difference approx. of derivative|
c |             (arrival time wrt each parameter)              |
c |     LAINTRFC Interface to LAPACK driver ssysvx.f which     |
c |              solves normal equations for each iteration    |
c |              Original code used a Gauss Elimination        |
c |              subroutine, copyright uncertain, so replaced  |
c |              with LAPACK for GNU license                   |
c |REQUIRES: LAPACK                                            |
c |http://www.netlib.org/lapack/                               |
c |                                                            |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
      parameter (nsmx=200,npmx=1,nmax=500)
c
      real*4            z(nsmx,npmx),xs(nsmx),zs(nsmx),xx(nsmx,npmx)
      real*4            t(nsmx,npmx)
      real*4            tc(nsmx,npmx),tp(nsmx,npmx),dt(nsmx,npmx)
      real*4            dv1(nsmx,npmx),dv2(nsmx,npmx),dv3(nsmx,npmx)
      real*4            dz1(nsmx,npmx),dz2(nsmx,npmx)
      real*4            aaa(5),dm(5,6),dparm(5)
      integer*4         npg(nsmx),ishcnt,newsht,ishw1
      character*4       typer(nsmx,npmx)
c
      real*4            s1(ndim)
      character*40      infil , outfil, aplc, outlst
      character*40      outls2
      character*40      title,xlab,ylab
      character*1024    cmd
      integer*1         bz
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bvsp'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +itmax,zmin,zmax)           
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
      n=ntrace
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itmax,zmin,zmax)           
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
      write(io11,'(''  '')')
      write(io11,901)
  901 format(5x,'Depth ',5x,'Angle ',5x,'V_Ave ',5x,'Pick  ',
     +5x,'Pick_V',5x,'Cos(0)'/)
c
c...read traces, compute/////////////////////////////////////////....
c
      jrec = 0
      ntr = 0
      ishw1=0
      ishcnt=0
      vbarmx=-999999.
      vbarmn=+999999.
      xgmx=-999999.
      xgmn=+999999.
      zsmx=-999999.
      zsm=+999999.
      zgmx=-999999.
      zgmn=+999999.
      xoffx=-999999.

c initialize npg array (required for win32)
      do 93, jrec=1,nsmx
      npg(jrec)=0
   93 continue
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      newsht=0
      if(ofrn.ne.ishw1) then
      ishw1=ofrn
      newsht=1
      endif
c
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
c
      irz=int(rz)
c
c...first break pick in sec.(mst=microsec, int*4)
      pick=float(mst)/1.E+06
      depth=sz-sd-rz
      angle=acos(depth/range)
      angle=angle*(180./3.1415926)
      if(pick.gt.0.0E+00) then
      vbar=range/pick
      else
      vbar=0.
      endif
      cosa=depth/range
      pickv=pick*cosa
      if(depth.le.zmax .and. depth.ge.zmin) then
      write(io11,989) depth, angle,vbar, pick, pickv, cosa
  989 format(5x, f6.2, 5x, f6.1, 5x, f6.0, 5x, f6.3, 5x, f6.3,5x,f6.4)
      endif
c
c...store max and min slant velocity update
      if(depth.le.zmax .and. depth.ge.zmin) then
      if(vbar.gt.vbarmx) vbarmx=vbar
      if(vbar.lt.vbarmn) vbarmn=vbar
      endif
c
c...assign variables for inversion
c...test for new shot
      if(newsht.eq.1) then
      ip=0
      ishcnt=ishcnt+1
      if(ishcnt.gt.nsmx) then
      write(*,*)'  ABORT--too many shots'
      stop
      endif
      xs(ishcnt)=RANGEX
      zs(ishcnt)=sz-sd
      endif
c
c-------included in interval zmin, zmax ?
      if(depth.le.zmax .and. depth.ge.zmin) then
      ip=ip+1
      if(ip.gt.npmx) then
      write(*,*)'  ABORT--too many receivers'
      stop
      endif
      xx(ishcnt,ip)=0.0
      z(ishcnt,ip)=rz
      t(ishcnt,ip)=pick
      npg(ishcnt)=ip
c
      if(xoffx.lt.RANGEX) xoffx=RANGEX
      if(RX.lt.xgmn) xgmn=RANGEX
      if(RX.gt.xgmx) xgmx=RANGEX
      endif
c-------end if zmin zmax test
c
c
c...END LOOP========================================....
  100 continue
c
      nshot=ishcnt
c
c...find datum (shallowest shot or geophone) NOTE: sz,rz are elevations
      datum=-99999999.
      do 40 is=1,nshot
      szd=zs(is)
      np=npg(is)
      if(szd.GT.datum)  datum=szd
      do 40 ip=1,np
      rz=z(is,ip)
      if(rz.GT.datum) datum=rz
   40 continue
c
c...apply datum to elevations
      zsm=0.
      do 41 is=1,nshot
      szd=zs(is)
      np=npg(is)
      zs(is)=datum-zs(is)
      szd=zs(is)
      do 41 ip=1,np
      z(is,ip)=datum-z(is,ip)
      rz=z(is,ip)
      if(rz.lt.zgmn) zgmn=rz
      if(rz.gt.zgmx) zgmx=rz
      if(szd.gt.zsmx) zsmx=szd
   41 continue
c
c...setup bargraph
      bar=float(itmax)/50.
      ibar=int(bar+0.5)
      invbar=int(1./bar+0.5)
c
c...Start inversion
      eps2=1.E-40
      flsqeh=1.E+20
      dampf=.1
ccc      itmax=40
c
c...set initial velocities
      v1=vbarmn
      v2=(vbarmx+vbarmn)/2.
      v3=vbarmx
c
      dzmx=zgmx-zgmn
      zmax=zgmx
      if(zmax.lt.zsmx) zmax=zsmx
c
c...set depth limits of search
      refrx=xoffx/4.
      if(zmax.lt.refrx) zmax=refrx
      zmin=zgmn
      if(zmin.gt.zsm) zmin=zsm
      if(zmin.lt.1.) zmin=1.
c
c...set initial thicknesses
      z1=.2*(zmax-zmin)+zmin
      z2=.4*(zmax-zmin)+zmin-z1
c
c...save current values
      v1h=v1
      v2h=v2
      v3h=v3
      z1h=z1
      z2h=z2
c
      iter=0
c
c...begin inversion loop
  520 continue
      call fstbrk(z,xx,z1,z2,v1,v2,v3,xs,zs,npg,nshot,typer,tc)
c
c...compute travel time error
      dte2=0.
      if(iter.ne.0) go to 398
      write(io11,9018)
 9018 format(//1x,'Iteration',8x,'LSQE',8x,'V1',8x,'V2',
     18x,'V3',8x,'Z1',8x,'Z2'/)
  398 continue
      do 400 is=1,nshot
      np=npg(is)
      do 401 ip=1,np
ccc   if(tc(is,ip).eq.0.) tc(is,ip)=t(is,ip)
      dt(is,ip)=t(is,ip)-tc(is,ip)
      dte2=dte2+dt(is,ip)*dt(is,ip)
  401 continue
  400 continue
      flsqe=sqrt(dte2/(float(n-1)))
c
c...test if lsqe increasing
      if(flsqe.gt.flsqeh) then
      v1=v1h
      v2=v2h
      v3=v3h
      z1=z1h
      z2=z2h
      flsqe=flsqeh
      endif
c
c...output
      write(io11,9017) iter,flsqe,v1,v2,v3,z1,z2
 9017 format(5x,i5,5x,f8.5,5(2x,f8.1))
c
c...compute derivatives
      pertb=.02
      z1p=z1+z1*pertb
      call fstbrk(z,xx,z1p,z2,v1,v2,v3,xs,zs,npg,nshot,typer,tp)
      call deriv(z1,z1p,dz1,npg,nshot,tc,tp)
c
      z2p=z2+z2*pertb
      call fstbrk(z,xx,z1,z2p,v1,v2,v3,xs,zs,npg,nshot,typer,tp)
      call deriv(z2,z2p,dz2,npg,nshot,tc,tp)
c
      v1p=v1+v1*pertb
      call fstbrk(z,xx,z1,z2,v1p,v2,v3,xs,zs,npg,nshot,typer,tp)
      call deriv(v1,v1p,dv1,npg,nshot,tc,tp)
c
      v2p=v2+v2*pertb
      call fstbrk(z,xx,z1,z2,v1,v2p,v3,xs,zs,npg,nshot,typer,tp)
      call deriv(v2,v2p,dv2,npg,nshot,tc,tp)
c
      v3p=v3+v3*pertb
      call fstbrk(z,xx,z1,z2,v1,v2,v3p,xs,zs,npg,nshot,typer,tp)
      call deriv(v3,v3p,dv3,npg,nshot,tc,tp)
c
c...compute summations and load system matrix
      do 440 j=1,5
      do 440 k=1,6
  440 dm(j,k)=0.
      do 460 is=1,nshot
      np=npg(is)
      do 460 ip=1,np
      aaa(1)=dv1(is,ip)
      aaa(2)=dv2(is,ip)
      aaa(3)=dv3(is,ip)
      aaa(4)=dz1(is,ip)
      aaa(5)=dz2(is,ip)
      do 480 im=1,5
      do 470 il=1,5
  470 dm(im,il)=dm(im,il)+aaa(im)*aaa(il)
      ilp=6
  480 dm(im,ilp)=dm(im,ilp)+dt(is,ip)*aaa(im)
  460 continue
c
c...avoid bomb if can't resolve parameter
      do 770 ja=1,5
      if(abs(dm(ja,ja)).lt.eps2) go to 900
  770 continue
c
c...save current values
      v1h=v1
      v2h=v2
      v3h=v3
      z1h=z1
      z2h=z2
      flsqeh=flsqe
c
c...determine parameter changes and update
c      call gauss(dm,dparm)
      call laintrfc(dm,dparm)
      v1=v1+dparm(1)*dampf
      v2=v2+dparm(2)*dampf
      v3=v3+dparm(3)*dampf
      z1=z1+dparm(4)*dampf
      z2=z2+dparm(5)*dampf
c
c...limit maximum or minimum values of parameters
      if((z1.lt.zmin).or.(z1.lt..1)) z1=z1-dparm(4)*dampf
      if(z2.lt.1.) z2=z2-dparm(5)*dampf
      if(v1.lt.50.) v1=50.
      if(v1.gt.4000.) v1=v1-dparm(1)*dampf
      if(v2.gt.4500.) v2=v2-dparm(2)*dampf
      if(v3.gt.5000.) v3=v3-dparm(3)*dampf
c
c...increasing velocity constraint
ccc   if(v2.lt.v1) v2=v1*1.01
ccc   if(v3.lt.v2) v3=v2*1.01
c
      if(z1.gt.zmax)z1=z1-dparm(4)*dampf
      z2z1=z2+z1
      if(z2z1.gt.zmax) z2=z2-dparm(5)*dampf
      z2z1=z2+z1
      if(z2z1.lt.zmax) go to 489
      z1=.4*zmax
      z2=.4*zmax
  489 continue
c
      v2mv1=v2-v1
      v3mv2=v3-v2
ccc   if(v2mv1.lt.70.) z1=2.
      if(v3mv2.lt.70.) z2=2.
      iter=iter+1
      iterm1=iter-1
c
ccc   write(io11,9017) iter,flsqe,v1,v2,v3,z1,z2
c
c...display progress....
      call pltbar(bar,invbar,ibar,iter,itmax)
c
      if (iter.gt.itmax) go to 900
      go to 520
  900 continue
      iter=iter-1
c
c...compute error
      dte2=0.
      write(io11,9019)
 9019 format(//8x,'G-Depth',3x,'XS-cord',2x,'T-obs',3x,'T-cal',
     +3x,'Del-T',3x,'TYPE'/)
c
c...open Octave plot file...
      io12=12
      write(outls2,'(a,''.m'')') outfil(1:8)
      open(io12,file=outls2,access='sequential')
      write(io12,*) '#Generated by bvsp.f                             '
      write(io12,*) 'clear'
      write(io12,*) 'data=[...'
c
c...output fit
      do 603 is=1,nshot
      np=npg(is)
      do 603 ip=1,np
      dt(is,ip)=t(is,ip)-tc(is,ip)
      dte2=dte2+dt(is,ip)*dt(is,ip)
c
c...listing file
      write(io11,9012) z(is,ip),xs(is),t(is,ip),tc(is,ip),dt(is,ip),
     +typer(is,ip)
 9012 format(5x,f8.1,3x,f8.1,3(2x,f6.4),2x,a4)
c
c...Octave file
      write(io12,9016)  z(is,ip),t(is,ip),tc(is,ip)
 9016 format(2x,f10.4,',  ',f10.5,',  ',f10.5,';')
  603 continue
      flsqe=sqrt(dte2/(float(n-1)))
      write(io11,9014) flsqe,iter
 9014 format(5x,'LSQE=',f8.3,5x,'Iteration=',i5)
      write(io11,'(''  N='',i5)') n
      write(io12,*) ' ;]'
      write(io12,*) 'z=data(:,1); tobs=data(:,2); tcal=data(:,3);'
      write(io12,*) 'plot(tobs,-z,"o");'
      write(io12,*) 'hold on                                           '
      write(io12,*) 'grid on                                           '
      write(io12,*) 'plot(tcal,-z,"-b");'
      write(io12,230) infil
  230 format("title('",a12,"')") 
      write(io12,231)
  231 format("xlabel('Time (s)')")
      write(io12,232)
  232 format("ylabel('Depth')")
c
c...gnuplot
      iofp=14
      open(iofp,file='bvsp.gp',access='sequential')
      call plot(nshot,npg,t,tc,z,iofp)
c
      bz=0
      write(cmd,'(''gnuplot bvsp.gp'',a)') bz
      call runcmd(cmd)
c
      write(*,'('' '')')
      write(6,240) ntr,outlst,outls2
240   format(' Traces Processed = ',i5/
     +' LISTING: file in ',a/
     +' OCTAVE: file in ',a)
      write(6,*) "GNUPLOT: bvsp.gp"
      write(6,*) "POSTSCRIPT: bvsp.ps"
c
c
      close(io11)
      close(io12)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: plot.f                                       |
c | DIRECTORY: BSUROOT/src/Fort/bsegy                          |
c | Gnuplot Version                                            |
c --------------------------------------------------------------
      subroutine plot(nshot,npg,t,tc,z,iofp)
      parameter (nsmx=100,npmx=1)
      real*4    t(nsmx,npmx),tc(nsmx,npmx),z(nsmx,npmx)
      integer*4 npg(nsmx)
      write(iofp,*) '#Generated by bvsp.f subroutine plot'
      write(iofp,*) "set terminal postscript size 7.0,5.0 in \"
      write(iofp,*) "portrait enhanced font 'Helvetica,10' "
      write(iofp,*) "set output 'bvsp.ps'                     "
      write(iofp,*) "set grid                                          "
      write(iofp,*) "set nokey                                         "
      write(iofp,*) "set xlabel 'Time (s)'                            "
      write(iofp,*) "set ylabel 'Depth'      "
c     write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
c     write(syrange,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax
      write(iofp,*) "set yrange [] reverse "
      write(iofp,*) "set mxtics 5                                      "
      write(iofp,*) "set mytics 5                                      "
      write(iofp,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(iofp,*) "plot '-' u ($1):($2) w p pt 6 ps 1, \"
      write(iofp,*) " '' u ($1):($2) w l ls 1     "
c...output fit
      do 10 is=1,nshot
      np=npg(is)
      do 10 ip=1,np
      write(iofp,9016)  t(is,ip),z(is,ip)
   10 continue
      write(iofp,*) "e "
      do 20 is=1,nshot
      np=npg(is)
      do 20 ip=1,np
      write(iofp,9016)  tc(is,ip),z(is,ip)
   20 continue
 9016 format(E14.7,E14.7)
      write(iofp,*) "e "
      close(iofp)
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: refr.f                                       |
c | DIRECTORY: BSUROOT/Fort/bsegy                              |
c |                                                            |
c |           Computes Refracted Direct Wave in bvsp.f         |
c |                                                            |
c | zg=geophone depth        xs=shot offset                    |
c | zs=shot depth            z=thickness of first layer        |
c | v1=first layer velocity  v2=second layer velocity          |
c | v3=third layer velocity  z2=thickness of second layer      |
c | tr=refracted direct arrival time                           |
c |                                                            |
c | if tr=98 no refraction calculated                          |
c | if tr=99 exeeded 1000 iterations in ray search             |
c |------------------------------------------------------------|
      subroutine refr(zg,z,z2,v1,v2,v3,tr,xss,zs)
      tatx=.9994
      ismx=100
      xs=abs(xss)
      zzz=z2+z
      tr=98.
      ptb=.06
      rad=3.1415926/180.
      amx=87.*rad
      amxm1=amx-1.*rad
      damp=.5
      tzg=zg
      tzs=zs
      if(zs.lt.zg) go to 5
      zg=tzs
      zs=tzg
    5 continue
c
c...Find ray
      if(zg.ge.zzz) go to 30
c
c...geophone in layer 2
      a2=atan(xs/(zg-zs))
      icount=0
   10 if(a2.ge.amx) a2=amx
      tat=((v1*sin(a2))/v2)
      if(tat.ge.tatx) tat=tatx
      a1=asin(tat)
      f=xs-(z-zs)*tan(a1)-(zg-z)*tan(a2)
      a2p=a2-ptb*rad
      tat=((v1*sin(a2p))/v2)
      if(tat.ge.tatx) tat=tatx
      a1p=asin(tat)
      fp=xs-(z-zs)*tan(a1p)-(zg-z)*tan(a2p)
      df=(f-fp)
      da=(a2-a2p)
      a2av=(a2+a2p)/2.
      fav=(f+fp)/2.
      a2=amx
      if(a2av.ge.amxm1) go to 20
      if(abs(df).le.0.1E-6) go to 20
      a2=a2av-(((fav)*da)/df)*damp
      a2=abs(a2)
      if(a2.ge.amx) a2=amx
      icount=icount+1
      if(icount.gt.ismx) go to 500
      if(abs(fav).lt.2.) go to 20
      go to 10
c
c...ray found
   20 if(a2.ge.amx)a2=amx
      tat=((v1*sin(a2))/v2)
      if(tat.ge.tatx) tat=tatx
      a1=asin(tat)
      d1=xs-(z-zs)*tan(a1)
      d2=zg-z
      ddd=sqrt(d1*d1+d2*d2)
      tr=(z-zs)/(cos(a1)*v1) + ddd/v2
      go to 400
c
c...geophone below layer 2
   30 a3=atan(xs/(zg-zs))
      icount=0
   40 if(a3.ge.amx) a3=amx
      tat=((v1*sin(a3))/v3)
      if(tat.ge.tatx) tat=tatx
      a1=asin(tat)
      tat=((v2*sin(a3))/v3)
      if(tat.ge.tatx) tat=tatx
      a2=asin(tat)
      f=xs-(z-zs)*tan(a1)-(zg-z-z2)*tan(a3)-z2*tan(a2)
      a3p=a3-ptb*rad
      tat=((v1*sin(a3p))/v3)
      if(tat.ge.tatx) tat=tatx
      a1p=asin(tat)
      tat=((v2*sin(a3p))/v3)
      if(tat.ge.tatx) tat=tatx
      a2p=asin(tat)
      fp=xs-(z-zs)*tan(a1p)-(zg-z-z2)*tan(a3p)-z2*tan(a2p)
      df=(f-fp)
      da=(a3-a3p)
      a3av=(a3+a3p)/2.
      fav=(f+fp)/2.
      a3=amx
      if(a3av.ge.amxm1) go to 50
      if(abs(df).le.0.1E-6) go to 50
      a3=a3av-(((fav)*da)/df)*damp
      a3=abs(a3)
      if(a3.ge.amx) a3=amx
      icount=icount+1
      if(icount.gt.ismx) go to 500
      if(abs(fav).lt.2.) go to 50
      go to 40
c
c...ray found
   50 if(a3.ge.amx) a3=amx
      tat=((v1*sin(a3))/v3)
      if(tat.ge.tatx) tat=tatx
      a1=asin(tat)
      tat=((v2*sin(a3))/v3)
      if(tat.ge.tatx) tat=tatx
      a2=asin(tat)
      d1=xs-(z-zs)*tan(a1)-z2*tan(a2)
      d2=zg-z-z2
      ddd=sqrt(d1*d1+d2*d2)
      tr=(z-zs)/(cos(a1)*v1)+z2/(cos(a2)*v2)+ddd/v3
      go to 400
c
c...source in layer 2, geophone in layer 3
      if (zg.le.zzz) go to 400
      a3=atan(xs/(zg-zs))
      icount=0
   70 if (a3.ge.amx) a3=amx
      tat=((v2*sin(a3))/v3)
      if(tat.ge.tatx) tat=tatx
      a2=asin(tat)
      f=xs-(zzz-zs)*tan(a2)-(zg-zzz)*tan(a3)
      a3p=a3-ptb*rad
      tat=((v2*sin(a3p))/v3)
      if(tat.ge.tatx) tat=tatx
      a2p=asin(tat)
      fp=xs-(zzz-zs)*tan(a2p)-(zg-zzz)*tan(a3p)
      df=(f-fp)
      da=(a3-a3p)
      a3av=(a3+a3p)/2.
      fav=(f+fp)/2.
      a3=amx
      if(a3av.ge.amxm1) go to 80
      if(abs(df).le.0.1E-2) go to 80
      a3=a3av-(((fav)*da)/df)*damp
      a3=abs(a3)
      if(a3.ge.amx) a3=amx
      icount=icount+1
      if(icount.gt.ismx) go to 500
      if(abs(fav).lt.2.0) go to 80
      go to 70
c
c...ray found
   80 if(a3.ge.amx) a3=amx
      tat=((v2*sin(a3))/v3)
      if(tat.ge.tatx) tat=tatx
      a2=asin(tat)
      d1=xs-(zzz-zs)*tan(a2)
      d2=zg-zzz
      ddd=sqrt(d1*d1+d2*d2)
      tr=(zzz-zs)/(cos(a2)*v2) + ddd/v3
      go to 400
 500  tr=99.
 400  zg=tzg
      zs=tzs
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: refh.f                                       |
c | DIRECTORY: BSUROOT/Fort/bsegy                              |
c |                                                            |
c |           Computes Refracted Head Wave in bvsp.f           |
c |                                                            |
c | zg=geophone depth        xs=shot offset                    |
c | zs=shot depth            z=thickness of first layer        |
c | v1=first layer velocity  v2=second layer velocity          |
c | v3=third layer velocity  z2=thickness of second layer      |
c | th=refracted head wave arrival time                        |
c |                                                            |
c | if th=98 no refraction calculated                          |
c |------------------------------------------------------------|
      subroutine refh(zg,z,z2,v1,v2,v3,th,th2,xs,zs)
      tzg=zg
      tzs=zs
      if(zs.lt.zg) go to 5
      zg=tzs
      zs=tzg
    5 continue
      th2=98.
      th=98.
      zzz=z+z2
      if(v2.le.v1) go to 400
      if(z.lt.zg) go to 400
c
c...head wave off 1st refractor z
      a1=asin(v1/v2)
      dxg=(z-zg)*tan(a1)
      dxs=(z-zs)*tan(a1)
      d=dxg+dxs
      if(xs.lt.d) go to 400
c
c...head wave possible
      th=((z-zs)*cos(a1))/v1 + ((z-zg)*cos(a1))/v1 + xs/v2
c
c...head wave off 2nd refractor z2
  400 if (v3.le.v2) go to 500
      if (zzz.lt.zg) go to 500
      if (zs.gt.zzz) go to 500
      if (zs.gt.z) go to 300
      a1=asin(v1/v3)
      a2=asin(v2/v3)
      x1=(z-zs)*tan(a1)
      x2=(z2)*tan(a2)
      if(zg.ge.z) go to 200
      x4=(z-zg)*tan(a1)
      x3=x2
      go to 220
  200 x3=((z+z2)-zg)*tan(a2)
      x4=0.
  220 d=x1+x2+x3+x4
      if(xs.lt.d) go to 500
c
c...head wave possible, 2nd layer
      th2=((z-zs)*cos(a1))/v1 + (z2*cos(a2))/v2 + xs/v3
      if (zg.ge.z) go to 240
      th2=th2 + (z2*cos(a2))/v2 + ((z-zg)*cos(a1))/v1
      go to 500
  240 th2=th2+ ((zzz-zg)*cos(a2))/v2
      go to 500
c
c...head wave off 2nd refractor, shot and rec. in layer 2
  300 a2=asin(v2/v3)
      dxg=(zzz-zg)*tan(a2)
      dxs=(zzz-zs)*tan(a2)
      d=dxg+dxs
      if(xs.lt.d) go to 500
c
c...head wave possible
      th=((zzz-zs)*cos(a2))/v2 + ((zzz-zg)*cos(a2))/v2 + xs/v3
  500 zg=tzg
      zs=tzs
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: direct.f                                     |
c | DIRECTORY: BSUROOT/Fort/bsegy                              |
c |                                                            |
c |            Computes Direct Wave (source and receiver in    |
c |                                  same layer only)          |
c |                                                            |
c | zg=geophone depth        xs=shot offset                    |
c | zs=shot depth            z=thickness of first layer        |
c | v1=first layer velocity  v2=second layer velocity          |
c | td=direct wave arrival time                                |
c |                                                            |
c | if td=98 no refraction calculated                          |
c |------------------------------------------------------------|
      subroutine direct(zg,z,z2,v1,v2,v3,td,xs,zs)
c
      dd=(zg-zs)*(zg-zs)+xs*xs
      td=98.
      zzz=z+z2
      if ( (zg.lt.z) .and. (zs.lt.z) ) go to 10
      if ( (zg.gt.z) .and. (zs.gt.z) ) go to 20
      return
c
c...source and receiver in layer 1
   10 td=sqrt(dd)/v1
      return
c
   20 if( (zg.gt.zzz) .and. (zs.gt.zzz) ) go to 40
      if( (zg.lt.z) .or. (zs.lt.z) ) return
      if( (zg.gt.zzz) .or. (zs.gt.zzz) ) return
c
c...source and receiver in layer 2
      td=sqrt(dd)/v2
      return
c
c...source and receiver in layer 3
   40 td=sqrt(dd)/v3
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: fstbrk.f                                     |
c | DIRECTORY: BSUROOT/Fort/bsegy                              |
c |                                                            |
c |            Determines first arrival and typer of wave       |
c |                                                            |
c | zg=geophone depth        xs=shot offset                    |
c | zs=shot depth            z=thickness of first layer        |
c | v1=first layer velocity  v2=second layer velocity          |
c | v3=third layer velocity                                    |
c | z1=thickness of upper layer                                |
c | z2=thickness of second layer                               |
c | nshot=number of shots                                      |
c | typer=type of wave                                          |
c | tc=calculated arrival time for wave                        |
c | npg=array with number of travel times for each shot        |
c | nshot=number of source efforts                             |
c |                                                            |
c | nsmx=number of shots maximum dimension                     |
c | npmx=number of picks per shot maximum dimension            |
c |------------------------------------------------------------|
      subroutine fstbrk(z,xx,z1,z2,v1,v2,v3,xs,zs,npg,nshot,typer,tc)
c
c...determine first arrival and type
c   (assuming 3 layer problem)
      parameter (nsmx=100,npmx=1)
c
      real*4            z(nsmx,npmx),xs(nsmx),zs(nsmx),xx(nsmx,npmx)
      real*4            tc(nsmx,npmx)
      integer*4         npg(nsmx)
      character*4       typer(nsmx,npmx),rf,dr,h1,h2,rnn
c
      data rf/'REFR'/, dr/'DRCT'/,h1/'HED1'/,h2/'HED2'/,rnn/'NONE'/
c
      io11=11
      do 10 is=1,nshot
      np=npg(is)
      do 10 ip=1,np
      zg=z(is,ip)
      xsi=abs(xs(is)-xx(is,ip))
      zsi=zs(is)
      tc(is,ip)=95.
      call refr(zg,z1,z2,v1,v2,v3,tr,xsi,zsi)
      call refh(zg,z1,z2,v1,v2,v3,th,th2,xsi,zsi)
      call direct(zg,z1,z2,v1,v2,v3,td,xsi,zsi)
c
c...determine first arrival
      if(tc(is,ip).lt.tr) go to 22
      tc(is,ip)=tr
      typer(is,ip)=rf
   22 if(tc(is,ip).lt.th) go to 24
      tc(is,ip)=th
      typer(is,ip)=h1
   24 if(tc(is,ip).lt.th2) go to 26
      tc(is,ip)=th2
      typer(is,ip)=h2
   26 if(tc(is,ip).lt.td) go to 9
      tc(is,ip)=td
      typer(is,ip)=dr
    9 if(tc(is,ip).ne.95.) go to 10
      tc(is,ip)=0.
      typer(is,ip)=rnn
   10 continue
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: deriv.f                                      |
c | DIRECTORY: BSUROOT/Fort/bsegy                              |
c |                                                            |
c |         Computes numerical derivatives (difference formula)|
c |                                                            |
c | dparm=derivative of travel time wrt. parameter parm        |
c | parm=parameter value                                       |
c | parmp=perturbed parameter                                  |
c | tc=calculated time                                         |
c | tp=perturbed time                                          |
c | nshot=number of shots                                      |
c | npg= number of picks per shot                              |
c --------------------------------------------------------------
      subroutine deriv(parm,parmp,dparm,npg,nshot,tc,tp)
c
      parameter (nsmx=100,npmx=1)
c
      real*4            tc(nsmx,npmx),tp(nsmx,npmx),dparm(nsmx,npmx)
      integer*4         npg(nsmx)
c
      do 10 is=1,nshot
      np=npg(is)
      do 10 ip=1,np
      dparm(is,ip)=(tc(is,ip)-tp(is,ip))/(parm-parmp)
   10 continue
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +itmax,zmin,zmax)           
c
      character*40      infil
      character*40      arg1,arg2
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bvsp: Travel time inversion of first     |'
      write(*,*) '| breaks.  (ray bending, horiz. layers VSP)|'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bvsp infile itmax zmin zmax'
      write(*,*)'infile:   =input file name '
      write(*,*)'itmax     =maximum number of iterations '
      write(*,*)'zmin      =minimum depth to include '
      write(*,*)'zmax      =maximum depth to include '
      write(*,*)' '
      stop    
      endif   
      endif   
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  Enter itmax, number of iterations'
      read(*,*) itmax
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') itmax
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  Enter zmin, minimum depth to include'
      read(*,*) zmin
      else
      call getarg(3,arg2)
      read(arg2,'(f8.0)') zmin 
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  Enter zmax, maximum depth to include'
      read(*,*) zmax
      else
      call getarg(4,arg2)
      read(arg2,'(f8.0)') zmax 
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itmax,zmin,zmax)           
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bvsp  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   itmax= '',i5)') itmax
      write(io11,'(''   zmin= '',f10.2)') zmin
      write(io11,'(''   zmax= '',f10.2)') zmax
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: laintrfc                                     |
c |------------------------------------------------------------|
c | linear algebra interface for LAPACK driver ssysvx          |
c | LAPACK is replacing original GAUSS Elimination Routine     |
c --------------------------------------------------------------
c
      subroutine laintrfc(a,x)
      parameter (ndim=5,nrhsx=1)
      real*4 a(ndim,*),af(ndim,ndim), b(ndim,nrhsx)
      real*4 ferr(nrhsx),berr(nrhsx),x(*),rcond
      character*1  uplo,fact
      integer      ipiv(ndim),iwork(ndim)
      real*4       work(ndim*3)
      io11=11
c
c...Program ssysvx  LAPACK symetric Ax=b
c              Least Squares Gm=d, then (G'G)m=(G'd)
c              and A=(G'G) symetric, b=(G'd), x=m for
c              this routine.
c In this case, input matrix, a is augmented, last column is G'd=b
c                             specifically, column 6 of a(5,6)
c
      uplo='U'
      fact='N'
c
c...yes, this is a waste of time, but the interface
c   is designed to make it look like the original call
c   to the gauss elimination, and this is a small problem
      do 10, j=1,ndim
10    b(j,nrhsx)=a(j,6)
c
      lda=ndim
      ldb=ndim
      ldx=ndim
      ldaf=ndim
      nrhs=nrhsx
      n=ndim
      lwork=ndim*3
c
c...call LAPACK driver
      call ssysvx(fact,uplo,n,nrhs,a,lda,af,ldaf,
     +ipiv,b,ldb,x,ldx,rcond,ferr,berr,work,lwork,iwork,info)
c
      if(info.ne.0) then
c     write(io11,*) 'WARNING: could be going singular here'
c     write(io11,*) 'LAPACK: info not zero, recip. cond # '
c     write(io11,'(2x,''info='',i5,$)') info
      write(io11,'(10x,''LAPACK: condition number='',e14.7)') rcond
      endif
c
      return
      end















