c |$Id: traplt.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $        |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: traplt.f                                     |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Line printer type plot of a seismic trace from a bsegy file|
c | Also plots spectrum, and a namelist of sample values       |
c | which can be used as a filter operator by program bftr.f   |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     TLIST   Generate line printer plot, time domain        |
c |     FLIST   Generates spectral line printer plot           |
c |                                                            |
c --------------------------------------------------------------
c
c
      parameter (ndim=32767)
      include '../include/bsegy.inc'
      real*4 s(ndim),v(ndim),xi(ndim),xr(ndim),amp(ndim),phz(ndim)
      complex x(ndim)
      character*50 infil,outlst
      character*70 title
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +ftmin,tmax,itrace,tzero,ilin,ndim,iunit1,s)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
      dt=fsamin
      itmin=int(ftmin/dt + 0.5) + 1
      itmax=int(tmax/dt + 0.5) + 1
      izero=int(tzero/dt + 0.5) + 1
c
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,outlst,
     +infil,ftmin,tmax,itrace,tzero,ilin,itmin,itmax,izero)
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
c...input  trace
      jrec=itrace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s,fsamin,infil,jrec,iexit)
c
      jtmp=0
      datamx=-1.0E+20
      datamn=1.0E+20
      do 75 jt=itmin,itmax
      jtmp=jtmp+1
      v(jtmp)=s(jt)
      if(s(jt).gt.datamx) datamx=s(jt)
      if(s(jt).lt.datamn) datamn=s(jt)
  75  continue
      range=datamx-datamn
      range=abs(range)*2.00
c
      write(title,900) ftmin,tmax,itrace,dt,infil(1:15)
 900  format('TMIN=',F7.3,' TMAX=',F7.3,' TRACE=',I3,
     +' DELT=',F7.5,' FILE: ',A)
      npts=jtmp
      call tlist(npts,v,title)
c...normalize data
      do 76 j=1,npts
      v(j)=v(j)/range
  76  continue
c
      nzph=int(tzero/dt+0.5)+1
      write(10,'(//2x,A/'' Trace data '')') title
      write(10,'(5x,''Normalization Factor='',e14.7//)') range
      write(10,'('' &filter'')')
      write(10,'(''  npf='',i5,'', nzph='',i5)') npts,nzph
      write(10,'(''  fil='')')
      write(10,901) (v(j),j=1,npts)
901   format(8(F9.4,','))
c
c...transform to frequency domain
      npt=npts-2
      call nrad2(npt,npr2)
      npr2=2*npr2
      if(npr2.gt.ndim) then
      write(*,'('' ABORT !!! Dimensions exceeded'')')
      stop
      endif
      do 80 j=1,npts
c...back off normalization 
      v(j)=v(j)*range
      x(j)=cmplx(v(j),0.0)
   80 continue
c
      if(npts.lt.npr2) then
      nptp1=npts+1
      do 85 j=nptp1,npr2
   85 x(j)=cmplx(0.0,0.0)
      endif
c
      sign=-1.0
      call nlogn(npr2,x,sign)
      call flist(npr2,x,title,fsamin,tzero,xr,xi,amp,phz,ilin)
c
      do 100, j=1,ntrace
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
100   continue
c
      write(*,*) '  '
      write(6,240) itrace
240   format(' Trace read = ',i5/
     +5x,'====> output in traplt.lst')
c
      close(iunit1)
      close(io11)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +ftmin,tmax,itrace,tzero,ilin,ndim,iunit1,s)
      include '../include/bsegy.inc'
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
      real*4            s(*)
      common /header/ hd
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '|traplt: Plots selected samples and the    |'
      write(*,*) '|spectrum for a seismic trace of bsegy data|'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'traplt infile tmin tmax trace# tzero ilin '
      write(*,*) ' '
      write(*,*) 'infile: =name of input file'
      write(*,*) 'tmin:   =start time in seconds '
      write(*,*) 'tmax:   =end time in seconds '
      write(*,*) 'trace#: =trace number to list and plot '
      write(*,*) 'tzero:=zero time for phase reference; spectral plot'
      write(*,*) 'ilin:  spectrum plot 1=linear 0=dB'
      write(*,*) ' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c...<<<<<<<<<<<<<<<< select data to plot>>>>>>>>>>>>>>>>>>>>
      if(nargsx.lt.(4+karg)) then
c
c...count traces and check dimensions------------------------
      icall1=0
      call CHKTRC(iunit1,icall1,npts,s,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
      close(iunit1)
c
      dt=fsamin
      write(6,220) npts  
220   format('  length= ',i5)
      write(6,221) dt
221   format('  sample rate=',f10.5)
      tmax=float(npts-1)*dt
      write(*,'(''  maximum trace time= '',f10.5)') tmax
      write(*,'(''  number of traces= '',i5)') ntrace
      write(*,*)' '
      write(*,*)' '
      write(*,'(''  enter tmin,tmax'')')
      read(*,*) ftmin,tmax
      else
      call getarg(2,arg2)
      read(arg2,'(f10.4)') ftmin
      call getarg(3,arg3)
      read(arg3,'(f10.4)') tmax
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,'(''  enter trace number'')')
      read(*,*) itrace
      else
      call getarg(4,arg4)
      read(arg4,'(i10)') itrace
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,'(''  enter tzero for spectral plot'')')
      read(*,*) tzero
      else
      call getarg(5,arg5)
      read(arg5,'(f10.4)') tzero
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,*)' Enter Spectral Plot Option:'
      write(*,*)'   1=Linear  0=dB'
      read(*,*) ilin
      else
      call getarg(6,arg6)
      read(arg6,'(i5)') ilin
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,outlst,
     +infil,ftmin,tmax,itrace,tzero,ilin,itmin,itmax,izero)
      character*40      outlst,infil
c
c...open output listing file...
      io11=10
      write(outlst,'(''traplt.lst'')') 
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program traplt'')')
      write(io11,'('' Input File: '',a)') infil
c
c      write(*,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   tmin, min_sample '',f10.2,5x,i5.5)')
     + ftmin,itmin
      write(io11,'(''   tmax, max_sample '',f10.2,5x,i5.5)')
     + tmax,itmax
      write(io11,'(''   itrace='',i5)') itrace
      write(io11,'(''  tzero, zero_sample'',f10.2,5x,i5.5)')
     + tzero,izero
      write(io11,'(''   ilin= '',i5)')    ilin      
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
