c |$Id: bnoise.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bnoise.f                                     |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | bnoise:  generates band pass limited noise                 |
c |                                                            |
c |  FUNCTIONS CALLED                                          |
c |  runif.f  CMLIB Function generates uniform distribution    |
c |           of random numbers (extends the period of the     |
c |           random number sequence.                          |
c |  rand.f   CMLIB Function called by runif.f                 |
c |  Both rand.f and runif.f are public domain.                |
c |------------------------------------------------------------|
c |  Random numbers are generated to create a random phase     |
c |  spectrum of the seismic trace.  The amplitude spectrum    |
c |  is flat across the specified band.  Since a radix 2 FFT   |
c |  is called (Robinson's nlogn.f), trace lengths other than  |
c |  a power of 2 will result in truncation of the inverse     |
c |  transform (which is the seismic trace output by bnoise).  |
c |  If you examine just one truncated trace, it will not have |
c |  a flat spectrum in the pass band.  However, if you stack  |
c |  the autocorrelations of all the traces, you will observe  |
c |  that the spectrum is generally quite flat in the pass-    |
c |  band (but it won't be perfect).  A test flow might be:    |
c |     bnos wave.seg .9812834672 20. 100. 10.                 |
c |     bxcr bnoswave.seg bnoswave.seg 0 .1 .1                 |
c |     bstk bxcrbnos.seg                                      |
c |     traplt bstkbxcr.seg 0. .1 1 .05 1                      |
c |------------------------------------------------------------|
c |  ARGUMENTS                                                 |
c |  v = trace containing noise                                |
c |  f1 = low frequency cut off                                |
c |  f2 = high frequency cut off                               |
c |  froll = frequency interval for roll off to zero amplitude |
c |  delt = sample rate                                        |
c |  npts = number of points in v                              |
c |  im = physical dimension (radix 2 algorithm needs more     |
c |       room than npts) of v and x                           |
c |  seed =initializes the random number function, rand.f      |
c |        Make seed a positive number less than 1             |
c --------------------------------------------------------------
c
      subroutine bnoise(npts,v,x,delt,f1,f2,froll,title,im,seed)
c  Lines that begin with c+ can be uncommented for debuging
c+    parameter (nnn=5096)
      external runif
      external rand
      real*4 v(*),table(66)
      complex x(*)
      character*70 title
      data itest/0/
c
c+    real*4 xr(nnn),xi(nnn),amp(nnn),phz(nnn)
c
c
c...find power of 2
      call nrad2(npts,npr2)
      if(npr2.gt.im) then
      write(*,'(''  abort--dimensions exceeded in bnoise'')')
      write(*,'(a)') title
      stop
      endif
      pi=3.1415926
c
c...set amplitude spectrum
      deltf=1.0/(float(npr2)*delt)
      do 10 j=1,npr2
  10  x(j)=cmplx(0.0,0.0)
      nfroll=int(froll/deltf+.5)
      if(nfroll.lt.1) nfroll=1
      df=pi/float(nfroll)
      if1=int(f1/deltf) + 1
      if(if1.lt.1) if1=1
      if2=int(f2/deltf) + 1
      if(if2.gt.npr2/2) if2=(npr2/2)
      k1=if1-nfroll
      if(k1.lt.1) k1=1
      k2=if2+nfroll
      if(k2.gt.(npr2/2)) k2=(npr2/2)
c
c...cosine taper
      do 15 j=1,nfroll
      arg=float(j-1)*df
      amp1=0.5*(1.00-cos(arg))
      x(k1)=cmplx(amp1,0.0)
      x(k2)=x(k1)
      k1=k1+1
      k2=k2-1
   15 continue
      do 20 j=if1,if2
      if(k2.lt.1) k2=1
      if(k1.gt.npr2/2) k1=npr2/2
      x(k1)=cmplx(1.00,0.00)
      x(k2)=x(k1)
      k1=k1+1
      k2=k2-1
   20 continue
c
c...generate random noise for phase
      factor=pi
c
c...initialize runif
      if(itest.eq.0) then
      itest=1
      w1=rand(abs(seed))
      w1=runif(table,32)
      endif
c
c...compute random phase
      npr=npr2/2
      do 30 kkk=1,npr
      w1=(runif(table,32)-0.5)*2.0
      v(kkk)=v(kkk)+w1*factor
   30 continue
c
c...combine random phase with amplitude
      k=npr2
      do 40 j=2,npr
      amp1=real(x(j))
      ximag=sin(v(j))*amp1
      xreal=cos(v(j))*amp1
      x(j)=cmplx(xreal,ximag)
      x(k)=cmplx(xreal,-ximag)
      k=k-1
   40 continue
      amp1=real(x(1))
      ximag=sin(v(1))*amp1
      xreal=cos(v(1))*amp1
      x(1)=cmplx(xreal,ximag)
c
c+    samin=delt
c+    tzero=0.0
c+    ilin=1
c+    call flist(npr2,x,title,samin,tzero,xr,xi,amp,phz,ilin)
c...take inverse transform -->to time domain
      sign=1.0
      call nlogn(npr2,x,sign)
c
      vmax=-1.0e+20
      vmin=+1.0e+20
      do 50 j=1,npts
      v(j)=real(x(j))
      if(v(j).gt.vmax) vmax=v(j)
      if(v(j).lt.vmin) vmin=v(j)
   50 continue
c
c...normalize to +/- 1
      range=abs(vmax-vmin)
      factor=2.0/range
      do 60 j=1,npts
      v(j)=v(j)*factor
   60 continue
c
c
      return
      end
